from web3 import Web3
from web3.providers.auto import (
    AutoProvider,
)
from web3.providers.eth_tester import (
    EthereumTesterProvider,
)


def test_set_providers(web3):
    providers = [EthereumTesterProvider()]

    web3.providers = providers

    assert web3.providers == providers


def test_set_providers_single(web3):
    providers = [EthereumTesterProvider()]

    web3.providers = providers[0]

    assert web3.providers == providers


def test_auto_provider_none():
    # init without provider succeeds, even when no provider available
    w3 = Web3()

    # non-node requests succeed
    w3.toHex(0) == '0x0'

    type(w3.providers[0]) == AutoProvider



import pytest

from eth_abi import (
    encode_abi,
)
from eth_utils import (
    encode_hex,
    remove_0x_prefix,
)


def test_contract_constructor_abi_encoding_with_no_constructor_fn(MathContract, MATH_CODE):
    deploy_data = MathContract._encode_constructor_data()
    assert deploy_data == MATH_CODE


def test_contract_constructor_abi_encoding_with_constructor_with_no_args(SimpleConstructorContract,
                                                                         SIMPLE_CONSTRUCTOR_CODE):
    deploy_data = SimpleConstructorContract._encode_constructor_data()
    assert deploy_data == SIMPLE_CONSTRUCTOR_CODE


@pytest.mark.parametrize(
    'arguments',
    (
        [],
        [1234],
        ['abcd', 1234],  # wrong order
        [1234, 'abcd', 'extra-arg'],  # extra arguments
        [-1234, 'abcd'],  # wrong types
        ['1234', 'abcd'],  # wrong types
    ),
)
def test_error_if_invalid_arguments_supplied(WithConstructorArgumentsContract, arguments):
    with pytest.raises(TypeError):
        WithConstructorArgumentsContract._encode_constructor_data(arguments)


@pytest.mark.parametrize(
    'bytes_arg',
    (
        b'abcd',
        '61626364',
        '0x61626364',
    ),
)
def test_contract_constructor_encoding_encoding(WithConstructorArgumentsContract, bytes_arg):
    deploy_data = WithConstructorArgumentsContract._encode_constructor_data([1234, bytes_arg])
    encoded_args = '0x00000000000000000000000000000000000000000000000000000000000004d26162636400000000000000000000000000000000000000000000000000000000'  # noqa: E501
    expected_ending = encode_hex(encode_abi(['uint256', 'bytes32'], [1234, b'abcd']))
    assert expected_ending == encoded_args
    assert deploy_data.endswith(remove_0x_prefix(expected_ending))



import pytest

from web3.providers.eth_tester import (
    EthereumTesterProvider,
)
from web3.utils.threads import (
    Timeout,
)


@pytest.fixture()
def filter_id(web3):
    if EthereumTesterProvider not in map(type, web3.providers):
        web3.providers = EthereumTesterProvider()

    block_filter = web3.eth.filter("latest")
    return block_filter.filter_id


def test_instatiate_existing_filter(web3, sleep_interval, wait_for_block, filter_id):
    with pytest.raises(TypeError):
        web3.eth.filter('latest', filter_id)
    with pytest.raises(TypeError):
        web3.eth.filter('latest', filter_id=filter_id)
    with pytest.raises(TypeError):
        web3.eth.filter(filter_params='latest', filter_id=filter_id)

    block_filter = web3.eth.filter(filter_id=filter_id)

    current_block = web3.eth.blockNumber

    wait_for_block(web3, current_block + 3)

    found_block_hashes = []
    with Timeout(5) as timeout:
        while len(found_block_hashes) < 3:
            found_block_hashes.extend(block_filter.get_new_entries())
            timeout.sleep(sleep_interval())

    assert len(found_block_hashes) == 3

    expected_block_hashes = [
        web3.eth.getBlock(n + 1).hash for n in range(current_block, current_block + 3)
    ]
    assert found_block_hashes == expected_block_hashes




import pytest
from unittest.mock import (
    Mock,
    patch,
)

from web3.datastructures import (
    AttributeDict,
)
from web3.middleware import (
    make_stalecheck_middleware,
)
from web3.middleware.stalecheck import (
    StaleBlockchain,
    _isfresh,
)


@pytest.fixture
def now():
    return 3141592653


@pytest.fixture
def allowable_delay():
    return 3 * 24 * 60 * 60


@pytest.fixture
def request_middleware(allowable_delay):
    middleware = make_stalecheck_middleware(allowable_delay)
    make_request, web3 = Mock(), Mock()
    initialized = middleware(make_request, web3)
    # for easier mocking, later:
    initialized.web3 = web3
    initialized.make_request = make_request
    return initialized


def stub_block(timestamp):
    return AttributeDict({
        'timestamp': timestamp,
        'number': 123,
    })


def test_is_not_fresh_with_no_block():
    assert not _isfresh(None, 1)


def test_is_not_fresh(now):
    with patch('time.time', return_value=now):
        SECONDS_ALLOWED = 2 * 86400
        stale = stub_block(now - SECONDS_ALLOWED - 1)
        assert not _isfresh(stale, SECONDS_ALLOWED)


def test_is_fresh(now):
    with patch('time.time', return_value=now):
        SECONDS_ALLOWED = 2 * 86400
        stale = stub_block(now - SECONDS_ALLOWED)
        assert _isfresh(stale, SECONDS_ALLOWED)


def test_stalecheck_pass(request_middleware):
    with patch('web3.middleware.stalecheck._isfresh', return_value=True):
        method, params = object(), object()
        request_middleware(method, params)
        request_middleware.make_request.assert_called_once_with(method, params)


def test_stalecheck_fail(request_middleware, now):
    with patch('web3.middleware.stalecheck._isfresh', return_value=False):
        request_middleware.web3.eth.getBlock.return_value = stub_block(now)
        with pytest.raises(StaleBlockchain):
            request_middleware('', [])


@pytest.mark.parametrize(
    'rpc_method',
    [
        'eth_getBlockByNumber',
    ]
)
def test_stalecheck_ignores_get_by_block_methods(request_middleware, rpc_method):
    # This is especially critical for getBlock('latest') which would cause infinite recursion
    with patch('web3.middleware.stalecheck._isfresh', side_effect=[False, True]):
        request_middleware(rpc_method, [])
        assert not request_middleware.web3.eth.getBlock.called


def test_stalecheck_calls_isfresh_with_empty_cache(request_middleware, allowable_delay):
    with patch('web3.middleware.stalecheck._isfresh', side_effect=[False, True]) as freshspy:
        block = object()
        request_middleware.web3.eth.getBlock.return_value = block
        request_middleware('', [])
        cache_call, live_call = freshspy.call_args_list
        assert cache_call[0] == (None, allowable_delay)
        assert live_call[0] == (block, allowable_delay)


def test_stalecheck_adds_block_to_cache(request_middleware, allowable_delay):
    with patch('web3.middleware.stalecheck._isfresh', side_effect=[False, True, True]) as freshspy:
        block = object()
        request_middleware.web3.eth.getBlock.return_value = block

        # cache miss
        request_middleware('', [])
        cache_call, live_call = freshspy.call_args_list
        assert freshspy.call_count == 2
        assert cache_call == ((None, allowable_delay), )
        assert live_call == ((block, allowable_delay), )

        # cache hit
        request_middleware('', [])
        assert freshspy.call_count == 3
        assert freshspy.call_args == ((block, allowable_delay), )



import pytest
from unittest.mock import (
    Mock,
)

from web3.middleware import (
    gas_price_strategy_middleware,
)


@pytest.fixture
def the_gas_price_strategy_middleware(web3):
    make_request, web3 = Mock(), Mock()
    initialized = gas_price_strategy_middleware(make_request, web3)
    initialized.web3 = web3
    initialized.make_request = make_request
    return initialized


def test_gas_price_generated(the_gas_price_strategy_middleware):
    the_gas_price_strategy_middleware.web3.eth.generateGasPrice.return_value = 5
    method = 'eth_sendTransaction'
    params = [{
        'to': '0x0',
        'value': 1,
    }]
    the_gas_price_strategy_middleware(method, params)
    the_gas_price_strategy_middleware.web3.eth.generateGasPrice.assert_called_once_with({
        'to': '0x0',
        'value': 1,
    })
    the_gas_price_strategy_middleware.make_request.assert_called_once_with(method, [{
        'to': '0x0',
        'value': 1,
        'gasPrice': 5,
    }])


def test_gas_price_not_overridden(the_gas_price_strategy_middleware):
    the_gas_price_strategy_middleware.web3.eth.generateGasPrice.return_value = 5
    method = 'eth_sendTransaction'
    params = [{
        'to': '0x0',
        'value': 1,
        'gasPrice': 10,
    }]
    the_gas_price_strategy_middleware(method, params)
    the_gas_price_strategy_middleware.make_request.assert_called_once_with(method, [{
        'to': '0x0',
        'value': 1,
        'gasPrice': 10,
    }])


def test_gas_price_not_set_without_gas_price_strategy(the_gas_price_strategy_middleware):
    the_gas_price_strategy_middleware.web3.eth.generateGasPrice.return_value = None
    method = 'eth_sendTransaction'
    params = [{
        'to': '0x0',
        'value': 1,
    }]
    the_gas_price_strategy_middleware(method, params)
    the_gas_price_strategy_middleware.make_request.assert_called_once_with(method, params)


def test_not_generate_gas_price_when_not_send_transaction_rpc(the_gas_price_strategy_middleware):
    the_gas_price_strategy_middleware.web3.getGasPriceStrategy = Mock()
    the_gas_price_strategy_middleware('eth_getBalance', [])
    the_gas_price_strategy_middleware.web3.getGasPriceStrategy.assert_not_called()



import pytest

from eth_utils import (
    is_same_address,
)

from web3.utils.events import (
    get_event_data,
)

# Ignore warning in pyethereum 1.6 - will go away with the upgrade
pytestmark = pytest.mark.filterwarnings("ignore:implicit cast from 'char *'")


@pytest.fixture()
def Emitter(web3, EMITTER):
    return web3.eth.contract(**EMITTER)


@pytest.fixture()
def emitter(web3, Emitter, wait_for_transaction, wait_for_block, address_conversion_func):
    wait_for_block(web3)
    deploy_txn_hash = Emitter.constructor().transact({'from': web3.eth.coinbase, 'gas': 1000000})
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn_hash)
    contract_address = address_conversion_func(deploy_receipt['contractAddress'])

    bytecode = web3.eth.getCode(contract_address)
    assert bytecode == Emitter.bytecode_runtime
    _emitter = Emitter(address=contract_address)
    assert _emitter.address == contract_address
    return _emitter


@pytest.mark.parametrize(
    'contract_fn,event_name,call_args,expected_args',
    (
        ('logNoArgs', 'LogAnonymous', [], {}),
        ('logNoArgs', 'LogNoArguments', [], {}),
        ('logSingle', 'LogSingleArg', [12345], {'arg0': 12345}),
        ('logSingle', 'LogSingleWithIndex', [12345], {'arg0': 12345}),
        ('logSingle', 'LogSingleAnonymous', [12345], {'arg0': 12345}),
        ('logDouble', 'LogDoubleArg', [12345, 54321], {'arg0': 12345, 'arg1': 54321}),
        ('logDouble', 'LogDoubleAnonymous', [12345, 54321], {'arg0': 12345, 'arg1': 54321}),
        ('logDouble', 'LogDoubleWithIndex', [12345, 54321], {'arg0': 12345, 'arg1': 54321}),
        (
            'logTriple',
            'LogTripleArg',
            [12345, 54321, 98765],
            {'arg0': 12345, 'arg1': 54321, 'arg2': 98765},
        ),
        (
            'logTriple',
            'LogTripleWithIndex',
            [12345, 54321, 98765],
            {'arg0': 12345, 'arg1': 54321, 'arg2': 98765},
        ),
        (
            'logQuadruple',
            'LogQuadrupleArg',
            [12345, 54321, 98765, 56789],
            {'arg0': 12345, 'arg1': 54321, 'arg2': 98765, 'arg3': 56789},
        ),
        (
            'logQuadruple',
            'LogQuadrupleWithIndex',
            [12345, 54321, 98765, 56789],
            {'arg0': 12345, 'arg1': 54321, 'arg2': 98765, 'arg3': 56789},
        ),
    )
)
def test_event_data_extraction(web3,
                               emitter,
                               wait_for_transaction,
                               emitter_log_topics,
                               emitter_event_ids,
                               contract_fn,
                               event_name,
                               call_args,
                               expected_args):
    function = getattr(emitter.functions, contract_fn)
    event_id = getattr(emitter_event_ids, event_name)
    txn_hash = function(event_id, *call_args).transact()
    txn_receipt = wait_for_transaction(web3, txn_hash)

    assert len(txn_receipt['logs']) == 1
    log_entry = txn_receipt['logs'][0]

    event_abi = emitter._find_matching_event_abi(event_name)

    event_topic = getattr(emitter_log_topics, event_name)
    is_anonymous = event_abi['anonymous']

    if is_anonymous:
        assert event_topic not in log_entry['topics']
    else:
        assert event_topic in log_entry['topics']

    event_data = get_event_data(event_abi, log_entry)

    assert event_data['args'] == expected_args
    assert event_data['blockHash'] == txn_receipt['blockHash']
    assert event_data['blockNumber'] == txn_receipt['blockNumber']
    assert event_data['transactionIndex'] == txn_receipt['transactionIndex']
    assert is_same_address(event_data['address'], emitter.address)
    assert event_data['event'] == event_name


def test_dynamic_length_argument_extraction(web3,
                                            emitter,
                                            wait_for_transaction,
                                            emitter_log_topics,
                                            emitter_event_ids):
    string_0 = "this-is-the-first-string-which-exceeds-32-bytes-in-length"
    string_1 = "this-is-the-second-string-which-exceeds-32-bytes-in-length"
    txn_hash = emitter.functions.logDynamicArgs(string_0, string_1).transact()
    txn_receipt = wait_for_transaction(web3, txn_hash)

    assert len(txn_receipt['logs']) == 1
    log_entry = txn_receipt['logs'][0]

    event_abi = emitter._find_matching_event_abi('LogDynamicArgs')

    event_topic = emitter_log_topics.LogDynamicArgs
    assert event_topic in log_entry['topics']

    string_0_topic = web3.sha3(text=string_0)
    assert string_0_topic in log_entry['topics']

    event_data = get_event_data(event_abi, log_entry)

    expected_args = {
        "arg0": string_0_topic,
        "arg1": string_1,
    }

    assert event_data['args'] == expected_args
    assert event_data['blockHash'] == txn_receipt['blockHash']
    assert event_data['blockNumber'] == txn_receipt['blockNumber']
    assert event_data['transactionIndex'] == txn_receipt['transactionIndex']
    assert is_same_address(event_data['address'], emitter.address)
    assert event_data['event'] == 'LogDynamicArgs'



import pytest
import socket

from web3.utils.module_testing import (
    EthModuleTest,
    NetModuleTest,
    PersonalModuleTest,
    VersionModuleTest,
    Web3ModuleTest,
)


def get_open_port():
    sock = socket.socket()
    sock.bind(('127.0.0.1', 0))
    port = sock.getsockname()[1]
    sock.close()
    return str(port)


class GoEthereumTest(Web3ModuleTest):
    def _check_web3_clientVersion(self, client_version):
        assert client_version.startswith('Geth/')


class GoEthereumEthModuleTest(EthModuleTest):
    def test_eth_replaceTransaction(self, web3, unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction(web3, unlocked_account)

    def test_eth_replaceTransaction_incorrect_nonce(self, web3, unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction_incorrect_nonce(web3, unlocked_account)

    def test_eth_replaceTransaction_gas_price_too_low(self, web3, unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction_gas_price_too_low(web3, unlocked_account)

    def test_eth_replaceTransaction_gas_price_defaulting_minimum(self, web3, unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction_gas_price_defaulting_minimum(web3, unlocked_account)

    def test_eth_replaceTransaction_gas_price_defaulting_strategy_higher(self,
                                                                         web3,
                                                                         unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction_gas_price_defaulting_strategy_higher(
            web3, unlocked_account
        )

    def test_eth_replaceTransaction_gas_price_defaulting_strategy_lower(self,
                                                                        web3,
                                                                        unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction_gas_price_defaulting_strategy_lower(
            web3, unlocked_account
        )

    def test_eth_modifyTransaction(self, web3, unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_modifyTransaction(web3, unlocked_account)


class GoEthereumVersionModuleTest(VersionModuleTest):
    pass


class GoEthereumNetModuleTest(NetModuleTest):
    pass


class GoEthereumPersonalModuleTest(PersonalModuleTest):
    pass



import pytest

from web3.utils.filters import (
    construct_event_filter_params,
)

EVENT_1_ABI = {
    "anonymous": False,
    "inputs": [
        {"indexed": False, "name": "arg0", "type": "uint256"},
        {"indexed": True, "name": "arg1", "type": "uint256"},
    ],
    "name": "Event_1",
    "type": "event",
}


@pytest.mark.parametrize(
    "event_abi,fn_kwargs,expected",
    (
        (EVENT_1_ABI, {}, {
            "topics": ['0xb470a829ed7792f06947f0ca3730a570cb378329ddcf09f2b4efabd6326f51f6'],
        }),
        (EVENT_1_ABI, {'topics': ['should-overwrite-topics']}, {
            "topics": [
                'should-overwrite-topics'
            ]
        }),
        (EVENT_1_ABI, {'contract_address': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601'}, {
            "topics": ['0xb470a829ed7792f06947f0ca3730a570cb378329ddcf09f2b4efabd6326f51f6'],
            'address': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601',
        }),
        (EVENT_1_ABI, {
            'contract_address': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601',
            'address': '0xBB9bc244D798123fDe783fCc1C72d3Bb8C189413',
        }, {
            "topics": ['0xb470a829ed7792f06947f0ca3730a570cb378329ddcf09f2b4efabd6326f51f6'],
            'address': [
                '0xBB9bc244D798123fDe783fCc1C72d3Bb8C189413',
                '0xd3CdA913deB6f67967B99D67aCDFa1712C293601',
            ],
        }),
        (EVENT_1_ABI, {'address': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601'}, {
            "topics": ['0xb470a829ed7792f06947f0ca3730a570cb378329ddcf09f2b4efabd6326f51f6'],
            'address': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601',
        }),
    ),
)
def test_construct_event_filter_params(event_abi, fn_kwargs, expected):
    _, actual = construct_event_filter_params(event_abi, **fn_kwargs)
    assert actual == expected


def hex_and_pad(i):
    unpadded_hex_value = hex(i).rstrip('L')
    return '0x' + unpadded_hex_value[2:].zfill(64)


@pytest.mark.parametrize(
    "event_abi,fn_kwargs,expected",
    (
        (EVENT_1_ABI, {}, [[]]),
        (EVENT_1_ABI, {'argument_filters': {'arg0': 1}}, [[hex_and_pad(1)]]),
        (EVENT_1_ABI, {'argument_filters': {'arg0': [1]}}, [[hex_and_pad(1)]]),
        (EVENT_1_ABI, {'argument_filters': {'arg0': [1, 2]}}, [
            [hex_and_pad(1)],
            [hex_and_pad(2)],
        ]),
    ),
)
def test_construct_event_filter_params_for_data_filters(event_abi, fn_kwargs,
                                                        expected):
    actual, _ = construct_event_filter_params(event_abi, **fn_kwargs)
    assert actual == expected



import pytest
import random

from flaky import (
    flaky,
)

from web3.utils.threads import (
    Timeout,
)


@pytest.mark.skip(reason="fixture 'web3_empty' not found")
@flaky(max_runs=3)
def test_sync_filter_against_log_events(web3_empty,
                                        emitter,
                                        wait_for_transaction,
                                        emitter_log_topics,
                                        emitter_event_ids
                                        ):
    web3 = web3_empty

    txn_filter = web3.eth.filter({})

    txn_hashes = []
    txn_hashes.append(emitter.functions.logNoArgs(emitter_event_ids.LogNoArguments).transact())

    for txn_hash in txn_hashes:
        wait_for_transaction(web3, txn_hash)

    with Timeout(5) as timeout:
        while not txn_filter.get_new_entries():
            timeout.sleep(random.random())

    seen_logs = txn_filter.get_new_entries()

    assert set(txn_hashes) == set(log['transactionHash'] for log in seen_logs)


@pytest.mark.skip(reason="fixture 'web3_empty' not found")
@flaky(max_runs=3)
def test_async_filter_against_log_events(web3_empty,
                                         emitter,
                                         wait_for_transaction,
                                         emitter_log_topics,
                                         emitter_event_ids
                                         ):
    web3 = web3_empty

    seen_logs = []
    txn_filter = web3.eth.filter({})
    txn_filter.watch(seen_logs.append)

    txn_hashes = []

    txn_hashes.append(emitter.functions.logNoArgs(emitter_event_ids.LogNoArguments).transact())

    for txn_hash in txn_hashes:
        wait_for_transaction(web3, txn_hash)

    with Timeout(5) as timeout:
        while not seen_logs:
            timeout.sleep(random.random())

    txn_filter.stop_watching(30)

    assert set(txn_hashes) == set(log['transactionHash'] for log in seen_logs)



import pytest

from web3.exceptions import (
    ValidationError,
)
from web3.middleware.simulate_unmined_transaction import (
    unmined_receipt_simulator_middleware,
)


@pytest.mark.parametrize(
    'make_chain_id, expect_success',
    (
        (
            lambda web3: web3.net.chainId,
            True,
        ),
        pytest.param(
            lambda web3: 999999999999,
            False,
            marks=pytest.mark.xfail,
        ),
    ),
)
def test_send_transaction_with_valid_chain_id(web3, make_chain_id, expect_success):
    transaction = {
        'to': web3.eth.accounts[1],
        'chainId': make_chain_id(web3),
    }
    if expect_success:
        # just be happy that we didn't crash
        web3.eth.sendTransaction(transaction)
    else:
        with pytest.raises(ValidationError) as exc_info:
            web3.eth.sendTransaction(transaction)

        assert 'chain ID' in str(exc_info.value)


def test_unmined_transaction_wait_for_receipt(web3, extra_accounts):
    web3.middleware_stack.add(unmined_receipt_simulator_middleware)
    txn_hash = web3.eth.sendTransaction({
        'from': web3.eth.coinbase,
        'to': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601',
        'value': 123457
    })
    assert web3.eth.getTransactionReceipt(txn_hash) is None

    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash)
    assert txn_receipt['transactionHash'] == txn_hash
    assert txn_receipt['blockHash'] is not None




from ens.utils import (
    init_web3,
)


def test_init_adds_middlewares():
    w3 = init_web3()
    middlewares = map(str, w3.manager.middleware_stack)
    assert 'stalecheck_middleware' in next(middlewares)



import pytest

from eth_utils import (
    is_integer,
)

from web3.contract import (
    ImplicitContract,
)


@pytest.fixture()
def math_contract(web3, MATH_ABI, MATH_CODE, MATH_RUNTIME, address_conversion_func):
    # Deploy math contract
    # NOTE Must use non-specialized contract factory or else deploy() doesn't work
    MathContract = web3.eth.contract(
        abi=MATH_ABI,
        bytecode=MATH_CODE,
        bytecode_runtime=MATH_RUNTIME,
    )
    tx_hash = MathContract.constructor().transact()
    tx_receipt = web3.eth.waitForTransactionReceipt(tx_hash)
    math_address = address_conversion_func(tx_receipt['contractAddress'])
    # Return interactive contract instance at deployed address
    # TODO Does parent class not implement 'deploy()' for a reason?
    MathContract = web3.eth.contract(
        abi=MATH_ABI,
        bytecode=MATH_CODE,
        bytecode_runtime=MATH_RUNTIME,
        ContractFactoryClass=ImplicitContract,
    )
    contract = MathContract(math_address)
    assert contract.address == math_address
    return contract


@pytest.fixture()
def get_transaction_count(web3):
    def get_transaction_count(blocknum_or_label):
        block = web3.eth.getBlock(blocknum_or_label)
        # Return the blocknum if we requested this via labels
        # so we can directly query the block next time (using the same API call)
        # Either way, return the number of transactions in the given block
        if blocknum_or_label in ["pending", "latest", "earliest"]:
            return block.number, len(block.transactions)
        else:
            return len(block.transactions)
    return get_transaction_count


def test_implicitcontract_call_default(math_contract, get_transaction_count):
    # When a function is called that defaults to call
    blocknum, starting_txns = get_transaction_count("pending")
    start_count = math_contract.counter()
    assert is_integer(start_count)
    # Check that a call was made and not a transact
    # (Auto-mining is enabled, so query by block number)
    assert get_transaction_count(blocknum) == starting_txns
    # Check that no blocks were mined
    assert get_transaction_count("pending") == (blocknum, 0)


def test_implicitcontract_transact_default(web3, math_contract, get_transaction_count):
    # Use to verify correct operation later on
    start_count = math_contract.counter()
    assert is_integer(start_count)  # Verify correct type
    # When a function is called that defaults to transact
    blocknum, starting_txns = get_transaction_count("pending")
    math_contract.increment(transact={})
    # Check that a transaction was made and not a call
    assert math_contract.counter() - start_count == 1
    # (Auto-mining is enabled, so query by block number)
    assert get_transaction_count(blocknum) == starting_txns + 1
    # Check that only one block was mined
    assert get_transaction_count("pending") == (blocknum + 1, 0)


def test_implicitcontract_call_override(math_contract, get_transaction_count):
    # When a function is called with transact override that defaults to call
    blocknum, starting_txns = get_transaction_count("pending")
    math_contract.counter(transact={})
    # Check that a transaction was made and not a call
    # (Auto-mining is enabled, so query by block number)
    assert get_transaction_count(blocknum) == starting_txns + 1
    # Check that only one block was mined
    assert get_transaction_count("pending") == (blocknum + 1, 0)


def test_implicitcontract_transact_override(math_contract, get_transaction_count):
    # Use to verify correct operation later on
    start_count = math_contract.counter()
    assert is_integer(start_count)  # Verify correct type
    # When a function is called with call override that defaults to transact
    blocknum, starting_txns = get_transaction_count("pending")
    math_contract.increment(call={})
    # Check that a call was made and not a transact
    assert math_contract.counter() - start_count == 0
    # (Auto-mining is enabled, so query by block number)
    assert get_transaction_count(blocknum) == starting_txns
    # Check that no blocks were mined
    assert get_transaction_count("pending") == (blocknum, 0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import (
    find_packages,
    setup,
)

extras_require = {
    'tester': [
        "eth-tester[py-evm]==0.1.0-beta.32",
        "py-geth>=2.0.1,<3.0.0",
    ],
    'testrpc': ["eth-testrpc>=1.3.3,<2.0.0"],
    'linter': [
        "flake8==3.4.1",
        "isort>=4.2.15,<5",
    ],
    'docs': [
        "mock",
        "sphinx-better-theme>=0.1.4",
        "click>=5.1",
        "configparser==3.5.0",
        "contextlib2>=0.5.4",
        #"eth-testrpc>=0.8.0",
        #"ethereum-tester-client>=1.1.0",
        "ethtoken",
        "py-geth>=1.4.0",
        "py-solc>=0.4.0",
        "pytest>=2.7.2",
        "sphinx",
        "sphinx_rtd_theme>=0.1.9",
        "toposort>=1.4",
        "urllib3",
        "web3>=2.1.0",
        "wheel"
    ],
    'dev': [
        "bumpversion",
        "flaky>=3.3.0",
        "hypothesis>=3.31.2",
        "pytest>=3.5.0,<4",
        "pytest-mock==1.*",
        "pytest-pythonpath>=0.3",
        "pytest-watch==4.*",
        "pytest-xdist==1.*",
        "setuptools>=36.2.0",
        "tox>=1.8.0",
        "tqdm",
        "when-changed"
    ]
}

extras_require['dev'] = (
    extras_require['tester'] +
    extras_require['linter'] +
    extras_require['docs'] +
    extras_require['dev']
)

setup(
    name='veriteos-web3',
    # *IMPORTANT*: Don't manually change the version here. Use the 'bumpversion' utility.
    version='4.7.1-4',
    description="""Veriteos Web3.py""",
    long_description_markdown_filename='README.md',
    author='Kyloon Chuah',
    author_email='kyloon@veriteos.com',
    url='https://github.com/veriteos/veriteos-web3.py',
    include_package_data=True,
    install_requires=[
        "toolz>=0.9.0,<1.0.0;implementation_name=='pypy'",
        "cytoolz>=0.9.0,<1.0.0;implementation_name=='cpython'",
        "eth-abi>=1.2.0,<2.0.0",
        "eth-account>=0.2.1,<0.4.0",
        "eth-utils>=1.0.1,<2.0.0",
        "hexbytes>=0.1.0,<1.0.0",
        "lru-dict>=1.1.6,<2.0.0",
        "eth-hash[pycryptodome]>=0.2.0,<1.0.0",
        "requests>=2.16.0,<3.0.0",
        "websockets>=6.0.0,<7.0.0",
        "pypiwin32>=223;platform_system=='Windows'",
    ],
    setup_requires=['setuptools-markdown'],
    python_requires='>=3.5, <4',
    extras_require=extras_require,
    py_modules=['web3', 'ens'],
    license="MIT",
    zip_safe=False,
    keywords='ethereum',
    packages=find_packages(exclude=["tests", "tests.*"]),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



import pytest

from eth_utils import (
    is_same_address,
)

# Ignore warning in pyethereum 1.6 - will go away with the upgrade
pytestmark = pytest.mark.filterwarnings("ignore:implicit cast from 'char *'")


@pytest.mark.parametrize('call_as_instance', (True, False))
def test_on_filter_using_get_all_entries_interface(
    web3,
    emitter,
    Emitter,
    wait_for_transaction,
    emitter_event_ids,
    call_as_instance,
):
    if call_as_instance:
        event_filter = emitter.events.LogNoArguments.createFilter(fromBlock='latest')
    else:
        event_filter = Emitter.events.LogNoArguments.createFilter(fromBlock='latest')

    txn_hash = emitter.functions.logNoArgs(emitter_event_ids.LogNoArguments).transact()
    wait_for_transaction(web3, txn_hash)

    log_entries = event_filter.get_all_entries()

    assert len(log_entries) == 1
    assert log_entries[0]['transactionHash'] == txn_hash

    # a second call still retrieves all results
    log_entries_2 = event_filter.get_all_entries()

    assert len(log_entries_2) == 1
    assert log_entries_2[0]['transactionHash'] == txn_hash


@pytest.mark.parametrize('call_as_instance', (True, False))
def test_get_all_entries_returned_block_data(
    web3,
    emitter,
    Emitter,
    wait_for_transaction,
    emitter_event_ids,
    call_as_instance,
):
    txn_hash = emitter.functions.logNoArgs(emitter_event_ids.LogNoArguments).transact()
    txn_receipt = wait_for_transaction(web3, txn_hash)

    if call_as_instance:
        contract = emitter
    else:
        contract = Emitter

    events = contract.events.LogNoArguments.createFilter(fromBlock=txn_receipt['blockNumber'])

    log_entries = events.get_all_entries()

    assert len(log_entries) == 1
    event_data = log_entries[0]
    assert event_data['args'] == {}
    assert event_data['blockHash'] == txn_receipt['blockHash']
    assert event_data['blockNumber'] == txn_receipt['blockNumber']
    assert event_data['transactionIndex'] == txn_receipt['transactionIndex']
    assert is_same_address(event_data['address'], emitter.address)
    assert event_data['event'] == 'LogNoArguments'



def test_snapshot_revert_to_latest_snapshot(web3):
    web3.testing.mine(5)

    block_before_snapshot = web3.eth.getBlock("latest")

    web3.testing.snapshot()

    block_after_snapshot = web3.eth.getBlock("latest")

    web3.testing.mine(3)

    block_after_mining = web3.eth.getBlock("latest")

    web3.testing.revert()

    block_after_revert = web3.eth.getBlock("latest")

    assert block_after_mining['number'] > block_before_snapshot['number']
    assert block_before_snapshot['hash'] == block_after_snapshot['hash']
    assert block_after_snapshot['hash'] == block_after_revert['hash']


def test_snapshot_revert_to_specific(web3):
    web3.testing.mine(5)

    block_before_snapshot = web3.eth.getBlock("latest")

    snapshot_idx = web3.testing.snapshot()

    block_after_snapshot = web3.eth.getBlock("latest")

    web3.testing.mine()
    web3.testing.snapshot()
    web3.testing.mine()
    web3.testing.snapshot()
    web3.testing.mine()
    web3.testing.snapshot()

    block_after_mining = web3.eth.getBlock("latest")

    web3.testing.revert(snapshot_idx)

    block_after_revert = web3.eth.getBlock("latest")

    assert block_after_mining['number'] > block_before_snapshot['number']
    assert block_before_snapshot['hash'] == block_after_snapshot['hash']
    assert block_after_snapshot['hash'] == block_after_revert['hash']



import pytest


@pytest.fixture(autouse=True)
def wait_for_first_block(web3, wait_for_block):
    wait_for_block(web3)


@pytest.fixture()
def math_contract(web3,
                  MATH_ABI,
                  MATH_CODE,
                  MATH_RUNTIME,
                  wait_for_transaction,
                  address_conversion_func):
    MathContract = web3.eth.contract(
        abi=MATH_ABI,
        bytecode=MATH_CODE,
        bytecode_runtime=MATH_RUNTIME,
    )
    deploy_txn = MathContract.constructor().transact({'from': web3.eth.coinbase})
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)

    assert deploy_receipt is not None
    contract_address = address_conversion_func(deploy_receipt['contractAddress'])
    web3.isAddress(contract_address)

    _math_contract = MathContract(address=contract_address)
    assert _math_contract.address == contract_address
    return _math_contract


@pytest.fixture()
def fallback_function_contract(web3,
                               FALLBACK_FUNCTION_ABI,
                               FALLBACK_FUNCTION_CODE,
                               FALLBACK_FUNCTION_RUNTIME,
                               wait_for_transaction,
                               address_conversion_func):
    fallback_contract = web3.eth.contract(
        abi=FALLBACK_FUNCTION_ABI,
        bytecode=FALLBACK_FUNCTION_CODE,
        bytecode_runtime=FALLBACK_FUNCTION_RUNTIME
    )
    deploy_txn = fallback_contract.constructor().transact({'from': web3.eth.coinbase})
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)

    assert deploy_receipt is not None
    contract_address = address_conversion_func(deploy_receipt['contractAddress'])
    web3.isAddress(contract_address)

    _fallback_function_contract = fallback_contract(address=contract_address)
    assert _fallback_function_contract.address == contract_address
    return _fallback_function_contract


def test_contract_estimateGas(web3, math_contract, estimateGas, transact):
    gas_estimate = estimateGas(contract=math_contract,
                               contract_function='increment')

    txn_hash = transact(
        contract=math_contract,
        contract_function='increment')

    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash)
    gas_used = txn_receipt.get('gasUsed')

    assert abs(gas_estimate - gas_used) < 21000


def test_contract_fallback_estimateGas(web3, fallback_function_contract):
    gas_estimate = fallback_function_contract.fallback.estimateGas()

    txn_hash = fallback_function_contract.fallback.transact()

    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash)
    gas_used = txn_receipt.get('gasUsed')

    assert abs(gas_estimate - gas_used) < 21000


def test_contract_estimateGas_with_arguments(web3, math_contract, estimateGas, transact):
    gas_estimate = estimateGas(contract=math_contract,
                               contract_function='add',
                               func_args=[5, 6])

    txn_hash = transact(
        contract=math_contract,
        contract_function='add',
        func_args=[5, 6])
    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash)
    gas_used = txn_receipt.get('gasUsed')

    assert abs(gas_estimate - gas_used) < 21000



def test_mining_property_tester(web3):
    assert web3.eth.mining is False


def test_mining_property_ipc_and_rpc(web3, wait_for_miner_start, skip_if_testrpc):
    skip_if_testrpc(web3)

    wait_for_miner_start(web3)
    assert web3.eth.mining is True



import json
import pytest

from web3.exceptions import (
    ValidationError,
)
from web3.utils.abi import (
    get_abi_input_types,
)
from web3.utils.function_identifiers import (
    FallbackFn,
)

SINGLE_FN_NO_ARGS = json.loads('[{"constant":false,"inputs":[],"name":"a","outputs":[],"type":"function"}]')  # noqa: E501
SINGLE_FN_ONE_ARG = json.loads('[{"constant":false,"inputs":[{"name":"","type":"uint256"}],"name":"a","outputs":[],"type":"function"}]')  # noqa: E501
MULTIPLE_FUNCTIONS = json.loads('[{"constant":false,"inputs":[],"name":"a","outputs":[],"type":"function"},{"constant":false,"inputs":[{"name":"","type":"bytes32"}],"name":"a","outputs":[],"type":"function"},{"constant":false,"inputs":[{"name":"","type":"uint256"}],"name":"a","outputs":[],"type":"function"},{"constant":false,"inputs":[{"name":"","type":"uint8"}],"name":"a","outputs":[],"type":"function"},{"constant":false,"inputs":[{"name":"","type":"int8"}],"name":"a","outputs":[],"type":"function"}]')  # noqa: E501
FALLBACK_FUNCTION = json.loads('[{"constant": false, "inputs": [], "name": "getData", "outputs": [{"name": "r", "type": "uint256"}], "payable": false, "stateMutability": "nonpayable", "type": "function"}, {"payable": false, "stateMutability": "nonpayable", "type": "fallback"}]')  # noqa: E501


def test_finds_single_function_without_args(web3):
    Contract = web3.eth.contract(abi=SINGLE_FN_NO_ARGS)

    abi = Contract._find_matching_fn_abi('a', [])
    assert abi['name'] == 'a'
    assert abi['inputs'] == []


def test_finds_single_function_with_args(web3):
    Contract = web3.eth.contract(abi=SINGLE_FN_ONE_ARG)

    abi = Contract._find_matching_fn_abi('a', [1234])
    assert abi['name'] == 'a'
    assert len(abi['inputs']) == 1
    assert abi['inputs'][0]['type'] == 'uint256'


def test_finds_fallback_function(web3):
    Contract = web3.eth.contract(abi=FALLBACK_FUNCTION)

    abi = Contract._find_matching_fn_abi(FallbackFn, [])
    assert abi['type'] == 'fallback'


def test_error_when_no_function_name_match(web3):
    Contract = web3.eth.contract(abi=SINGLE_FN_NO_ARGS)

    with pytest.raises(ValidationError):
        Contract._find_matching_fn_abi('no_function_name', [1234])


@pytest.mark.parametrize(
    'arguments,expected_types',
    (
        ([], []),
        ([b'arst'], ['bytes32']),
        (['0xf00b47'], ['bytes32']),
        ([1234567890], ['uint256']),
        # ([255], ['uint8']),  # TODO: enable
        ([-1], ['int8']),
    )
)
def test_finds_function_with_matching_args(web3, arguments, expected_types):
    Contract = web3.eth.contract(abi=MULTIPLE_FUNCTIONS)

    abi = Contract._find_matching_fn_abi('a', arguments)
    assert abi['name'] == 'a'
    assert len(abi['inputs']) == len(expected_types)
    assert set(get_abi_input_types(abi)) == set(expected_types)


def test_error_when_duplicate_match(web3):
    Contract = web3.eth.contract(abi=MULTIPLE_FUNCTIONS)

    with pytest.raises(ValidationError):
        Contract._find_matching_fn_abi('a', [100])



import asyncio
import time

import websockets


async def wait_for_ws(endpoint_uri, event_loop, timeout=60):
    start = time.time()
    while time.time() < start + timeout:
        try:
            async with websockets.connect(uri=endpoint_uri, loop=event_loop):
                pass
        except (ConnectionRefusedError, OSError):
            await asyncio.sleep(0.01)
        else:
            break



import asyncio
import pytest

from web3.utils.module_testing.emitter_contract import (
    EMITTER_ABI,
    EMITTER_BYTECODE,
)
from web3.utils.module_testing.math_contract import (
    MATH_ABI,
    MATH_BYTECODE,
)


@pytest.fixture(scope="module")
def math_contract_factory(web3):
    contract_factory = web3.eth.contract(abi=MATH_ABI, bytecode=MATH_BYTECODE)
    return contract_factory


@pytest.fixture(scope="module")
def emitter_contract_factory(web3):
    contract_factory = web3.eth.contract(abi=EMITTER_ABI, bytecode=EMITTER_BYTECODE)
    return contract_factory


@pytest.yield_fixture(scope="module")
def event_loop(request):
    loop = asyncio.get_event_loop_policy().new_event_loop()
    yield loop
    loop.close()



import pytest
from unittest.mock import (
    Mock,
)

from eth_utils import (
    decode_hex,
)

from web3.contract import (
    CONCISE_NORMALIZERS,
    ConciseContract,
    ConciseMethod,
)


def deploy(web3, Contract, args=None):
    args = args or []
    deploy_txn = Contract.constructor(*args).transact()
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert deploy_receipt is not None
    contract = Contract(address=deploy_receipt['contractAddress'])
    assert len(web3.eth.getCode(contract.address)) > 0
    return contract


@pytest.fixture()
def EMPTY_ADDR(address_conversion_func):
    addr = '0x' + '00' * 20
    return address_conversion_func(addr)


@pytest.fixture()
def zero_address_contract(web3, WithConstructorAddressArgumentsContract, EMPTY_ADDR):
    deploy_txn = WithConstructorAddressArgumentsContract.constructor(
        EMPTY_ADDR,
    ).transact()
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert deploy_receipt is not None
    _address_contract = WithConstructorAddressArgumentsContract(
        address=deploy_receipt['contractAddress'],
    )
    return ConciseContract(_address_contract)


def test_concisecontract_call_default():
    mock = Mock()
    sweet_method = ConciseMethod(mock.functions.grail)
    sweet_method(1, 2)
    mock.functions.grail.assert_called_once_with(1, 2)
    # Checking in return_value, ie the function instance
    mock.functions.grail.return_value.call.assert_called_once_with({})


def test_concisecontract_custom_transact():
    mock = Mock()
    sweet_method = ConciseMethod(mock.functions.grail)
    sweet_method(1, 2, transact={'holy': 3})
    mock.functions.grail.assert_called_once_with(1, 2)
    # Checking in return_value, ie the function instance
    mock.functions.grail.return_value.transact.assert_called_once_with({'holy': 3})


def test_concisecontract_two_keywords_fail():
    mock = Mock()
    sweet_method = ConciseMethod(mock)
    with pytest.raises(TypeError):
        sweet_method(1, 2, transact={'holy': 3}, call={'count_to': 4})


def test_concisecontract_unknown_keyword_fails():
    contract = Mock()
    sweet_method = ConciseMethod(contract.functions.grail)
    with pytest.raises(TypeError):
        sweet_method(1, 2, count={'to': 5})


def test_concisecontract_returns_none_for_0addr(zero_address_contract):
    result = zero_address_contract.testAddr()
    assert result is None


def test_class_construction_sets_class_vars(web3,
                                            MATH_ABI,
                                            MATH_CODE,
                                            MATH_RUNTIME,
                                            some_address,
                                            ):
    MathContract = web3.eth.contract(
        abi=MATH_ABI,
        bytecode=MATH_CODE,
        bytecode_runtime=MATH_RUNTIME,
    )

    classic = MathContract(some_address)
    assert classic.web3 == web3
    assert classic.bytecode == decode_hex(MATH_CODE)
    assert classic.bytecode_runtime == decode_hex(MATH_RUNTIME)


def test_conciscecontract_keeps_custom_normalizers_on_base(web3):
    base_contract = web3.eth.contract()
    # give different normalizers to this base instance
    base_contract._return_data_normalizers = base_contract._return_data_normalizers + tuple([None])

    # create concisce contract with custom contract
    new_normalizers_size = len(base_contract._return_data_normalizers)
    concise = ConciseContract(base_contract)

    # check that concise contract includes the new normalizers
    concise_normalizers_size = len(concise._classic_contract._return_data_normalizers)
    assert concise_normalizers_size == new_normalizers_size + len(CONCISE_NORMALIZERS)
    assert concise._classic_contract._return_data_normalizers[0] is None


def test_conciscecontract_function_collision(
        web3,
        StringContract):

    contract = deploy(web3, StringContract, args=["blarg"])

    def getValue():
        assert 'getValue' in [
            item['name'] for item
            in StringContract.abi
            if 'name' in item]

    setattr(ConciseContract, 'getValue', getValue)

    concise_contract = ConciseContract(contract)

    assert isinstance(concise_contract, ConciseContract)

    with pytest.raises(AttributeError, match=r'Namespace collision .* with ConciseContract API.'):
        concise_contract.getValue()



import pytest

import eth_account
import eth_keys
from eth_utils import (
    to_bytes,
    to_hex,
)
from hexbytes import (
    HexBytes,
)

from web3 import Web3
from web3.exceptions import (
    InvalidAddress,
)
from web3.middleware import (
    construct_result_generator_middleware,
    construct_sign_and_send_raw_middleware,
)
from web3.middleware.signing import (
    gen_normalized_accounts,
)
from web3.providers import (
    BaseProvider,
)
from web3.providers.eth_tester import (
    EthereumTesterProvider,
)
from web3.utils.toolz import (
    identity,
    merge,
    valfilter,
)

PRIVATE_KEY_1 = to_bytes(
    hexstr='0x6a8b4de52b288e111c14e1c4b868bc125d325d40331d86d875a3467dd44bf829')

ADDRESS_1 = '0x634743b15C948820069a43f6B361D03EfbBBE5a8'

PRIVATE_KEY_2 = to_bytes(
    hexstr='0xbf963e13b164c2100795f53e5590010f76b7a91b5a78de8e2b97239c8cfca8e8')

ADDRESS_2 = '0x91eD14b5956DBcc1310E65DC4d7E82f02B95BA46'

KEY_FUNCS = (
    eth_keys.keys.PrivateKey,
    eth_account.Account.privateKeyToAccount,
    HexBytes,
    to_hex,
    identity,
)


SAME_KEY_MIXED_TYPE = tuple(key_func(PRIVATE_KEY_1) for key_func in KEY_FUNCS)

MIXED_KEY_MIXED_TYPE = tuple(
    key_func(key) for key in [PRIVATE_KEY_1, PRIVATE_KEY_2] for key_func in KEY_FUNCS
)

SAME_KEY_SAME_TYPE = (
    eth_keys.keys.PrivateKey(PRIVATE_KEY_1),
    eth_keys.keys.PrivateKey(PRIVATE_KEY_1)
)

MIXED_KEY_SAME_TYPE = (
    eth_keys.keys.PrivateKey(PRIVATE_KEY_1), eth_keys.keys.PrivateKey(PRIVATE_KEY_2)
)


class DummyProvider(BaseProvider):
    def make_request(self, method, params):
        raise NotImplementedError("Cannot make request for {0}:{1}".format(
            method,
            params,
        ))


@pytest.fixture()
def result_generator_middleware():
    return construct_result_generator_middleware({
        'eth_sendRawTransaction': lambda *args: args,
        'net_version': lambda *_: 1,
    })


@pytest.fixture()
def w3_base():
    return Web3(providers=[DummyProvider()], middlewares=[])


@pytest.fixture()
def w3_dummy(w3_base, result_generator_middleware):
    w3_base.middleware_stack.add(result_generator_middleware)
    return w3_base


@pytest.mark.parametrize(
    'method,key_object,from_,expected',
    (
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE, ADDRESS_2, NotImplementedError),
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE, ADDRESS_1, 'eth_sendRawTransaction'),
        ('eth_sendTransaction', MIXED_KEY_MIXED_TYPE, ADDRESS_2, 'eth_sendRawTransaction'),
        ('eth_sendTransaction', MIXED_KEY_MIXED_TYPE, ADDRESS_1, 'eth_sendRawTransaction'),
        ('eth_sendTransaction', SAME_KEY_SAME_TYPE, ADDRESS_2, NotImplementedError),
        ('eth_sendTransaction', SAME_KEY_SAME_TYPE, ADDRESS_1, 'eth_sendRawTransaction'),
        ('eth_sendTransaction', MIXED_KEY_SAME_TYPE, ADDRESS_2, 'eth_sendRawTransaction'),
        ('eth_sendTransaction', MIXED_KEY_SAME_TYPE, ADDRESS_1, 'eth_sendRawTransaction'),
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE[0], ADDRESS_1, 'eth_sendRawTransaction'),
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE[1], ADDRESS_1, 'eth_sendRawTransaction'),
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE[2], ADDRESS_1, 'eth_sendRawTransaction'),
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE[3], ADDRESS_1, 'eth_sendRawTransaction'),
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE[4], ADDRESS_1, 'eth_sendRawTransaction'),
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE[0], ADDRESS_2, NotImplementedError),
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE[1], ADDRESS_2, NotImplementedError),
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE[2], ADDRESS_2, NotImplementedError),
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE[3], ADDRESS_2, NotImplementedError),
        ('eth_sendTransaction', SAME_KEY_MIXED_TYPE[4], ADDRESS_2, NotImplementedError),
        ('eth_call', MIXED_KEY_MIXED_TYPE, ADDRESS_1, NotImplementedError),
    )
)
def test_sign_and_send_raw_middleware(
        w3_dummy,
        w3,
        method,
        from_,
        expected,
        key_object):
    w3_dummy.middleware_stack.add(
        construct_sign_and_send_raw_middleware(key_object))

    if isinstance(expected, type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            w3_dummy.manager.request_blocking(
                method,
                [{
                    'to': '0x7E5F4552091A69125d5DfCb7b8C2659029395Bdf',
                    'from': from_,
                    'gas': 21000,
                    'gasPrice': 0,
                    'value': 1,
                    'nonce': 0
                }])
    else:
        actual = w3_dummy.manager.request_blocking(
            method,
            [{
                'to': '0x7E5F4552091A69125d5DfCb7b8C2659029395Bdf',
                'from': from_,
                'gas': 21000,
                'gasPrice': 0,
                'value': 1,
                'nonce': 0
            }])
        raw_txn = actual[1][0]
        actual_method = actual[0]
        assert actual_method == expected
        assert isinstance(raw_txn, bytes)


@pytest.fixture()
def w3():
    return Web3(EthereumTesterProvider())


@pytest.mark.parametrize(
    'key_object',
    (
        (SAME_KEY_MIXED_TYPE),
        (MIXED_KEY_MIXED_TYPE),
        (SAME_KEY_SAME_TYPE),
        (MIXED_KEY_SAME_TYPE),
        (SAME_KEY_MIXED_TYPE[0]),
        (SAME_KEY_MIXED_TYPE[1]),
        (SAME_KEY_MIXED_TYPE[2]),
        (SAME_KEY_MIXED_TYPE[3]),
        (SAME_KEY_MIXED_TYPE[4]),
    )
)
def test_gen_normalized_accounts(key_object):
    accounts = gen_normalized_accounts(key_object)
    assert all(isinstance(account, eth_account.local.LocalAccount) for account in accounts.values())


def test_gen_normalized_accounts_type_error(w3):
    with pytest.raises(TypeError):
        gen_normalized_accounts(1234567890)


@pytest.fixture()
def fund_account(w3):
    # fund local account
    tx_value = w3.toWei(10, 'ether')
    for address in (ADDRESS_1, ADDRESS_2):
        w3.eth.sendTransaction({
            'to': address,
            'from': w3.eth.accounts[0],
            'gas': 21000,
            'value': tx_value})
        assert w3.eth.getBalance(address) == tx_value


@pytest.mark.parametrize(
    'transaction,expected,key_object,from_',
    (
        (
            #  Transaction with set gas
            {
                'gas': 21000,
                'gasPrice': 0,
                'value': 1
            },
            -1,
            MIXED_KEY_MIXED_TYPE,
            ADDRESS_1,
        ),
        (
            #  Transaction with no set gas
            {
                'value': 1
            },
            -1,
            MIXED_KEY_MIXED_TYPE,
            ADDRESS_1,
        ),
        (
            # Transaction with mismatched sender
            # expect a key error with sendTransaction + unmanaged account
            {
                'gas': 21000,
                'value': 10
            },
            KeyError,
            SAME_KEY_MIXED_TYPE,
            ADDRESS_2,
        ),
        (
            #  Transaction with invalid sender
            {
                'gas': 21000,
                'value': 10
            },
            InvalidAddress,
            SAME_KEY_MIXED_TYPE,
            '0x0000',
        )
    )
)
def test_signed_transaction(
        w3,
        fund_account,
        transaction,
        expected,
        key_object,
        from_):
    w3.middleware_stack.add(construct_sign_and_send_raw_middleware(key_object))

    # Drop any falsy addresses
    to_from = valfilter(bool, {'to': w3.eth.accounts[0], 'from': from_})

    _transaction = merge(transaction, to_from)

    if isinstance(expected, type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            start_balance = w3.eth.getBalance(_transaction.get('from', w3.eth.accounts[0]))
            w3.eth.sendTransaction(_transaction)
    else:
        start_balance = w3.eth.getBalance(_transaction.get('from', w3.eth.accounts[0]))
        w3.eth.sendTransaction(_transaction)
        assert w3.eth.getBalance(_transaction.get('from')) <= start_balance + expected



import codecs
import itertools
import pytest
import time
import uuid

from eth_utils import (
    is_integer,
    to_tuple,
)

from web3 import Web3
from web3.middleware import (  # noqa: F401
    construct_error_generator_middleware,
    construct_latest_block_based_cache_middleware,
    construct_result_generator_middleware,
)
from web3.providers.base import (
    BaseProvider,
)
from web3.utils.caching import (
    generate_cache_key,
)


@pytest.fixture
def w3_base():
    return Web3(providers=[BaseProvider()], middlewares=[])


def _mk_block(n, timestamp):
    return {
        'hash': codecs.decode(str(n).zfill(32), 'hex'),
        'number': n,
        'timestamp': timestamp,
    }


@to_tuple
def generate_block_history(num_mined_blocks=5, block_time=1):
    genesis = _mk_block(0, time.time())
    yield genesis
    for block_number in range(1, num_mined_blocks + 1):
        yield _mk_block(
            block_number,
            genesis['timestamp'] + 2 * block_number,
        )


@pytest.fixture
def construct_block_data_middleware():
    def _construct_block_data_middleware(num_blocks):
        blocks = generate_block_history(num_blocks)
        _block_info = {
            'blocks': blocks,
            'head_block_number': blocks[0]['number']
        }

        def _evm_mine(method, params, block_info=_block_info):
            num_blocks = params[0]
            head_block_number = block_info['head_block_number']
            if head_block_number + num_blocks >= len(block_info['blocks']):
                raise ValueError("no more blocks to mine")

            block_info['head_block_number'] += num_blocks

        def _get_block_by_number(method, params, block_info=_block_info):
            block_id = params[0]
            blocks = block_info['blocks']
            head_block_number = block_info['head_block_number']

            if block_id == 'latest':
                return blocks[head_block_number]
            elif block_id == 'pending':
                if head_block_number + 1 >= len(blocks):
                    raise ValueError("no pending block")
                return blocks[head_block_number + 1]
            elif block_id == 'earliest':
                return blocks[0]
            elif is_integer(block_id):
                if block_id <= head_block_number:
                    return blocks[block_id]
                else:
                    return None
            else:
                raise TypeError('Invalid type for block_id')

        def _get_block_by_hash(method, params, block_info=_block_info):
            block_hash = params[0]
            blocks = block_info['blocks']
            head_block_number = block_info['head_block_number']

            blocks_by_hash = {
                block['hash']: block
                for block
                in blocks
            }
            try:
                block = blocks_by_hash[block_hash]
                if block['number'] <= head_block_number:
                    return block
                else:
                    return None
            except KeyError:
                return None

        return construct_result_generator_middleware({
            'eth_getBlockByNumber': _get_block_by_number,
            'eth_getBlockByHash': _get_block_by_hash,
            'evm_mine': _evm_mine,
        })
    return _construct_block_data_middleware


@pytest.fixture
def block_data_middleware(construct_block_data_middleware):
    return construct_block_data_middleware(5)


@pytest.fixture
def result_generator_middleware():
    return construct_result_generator_middleware({
        'fake_endpoint': lambda *_: str(uuid.uuid4()),
        'not_whitelisted': lambda *_: str(uuid.uuid4()),
    })


@pytest.fixture
def latest_block_based_cache_middleware():
    return construct_latest_block_based_cache_middleware(
        cache_class=dict,
        average_block_time_sample_size=1,
        default_average_block_time=0.1,
        rpc_whitelist={'fake_endpoint'},
    )


@pytest.fixture
def w3(w3_base,
       result_generator_middleware,
       block_data_middleware,
       latest_block_based_cache_middleware):
    w3_base.middleware_stack.add(block_data_middleware)
    w3_base.middleware_stack.add(result_generator_middleware)
    w3_base.middleware_stack.add(latest_block_based_cache_middleware)
    return w3_base


def test_latest_block_based_cache_middleware_pulls_from_cache(
        w3_base,
        block_data_middleware,
        result_generator_middleware):
    w3 = w3_base
    w3.middleware_stack.add(block_data_middleware)
    w3.middleware_stack.add(result_generator_middleware)

    current_block_hash = w3.eth.getBlock('latest')['hash']

    def cache_class():
        return {
            generate_cache_key(
                (current_block_hash, 'fake_endpoint', [1])
            ): {'result': 'value-a'},
        }

    w3.middleware_stack.add(construct_latest_block_based_cache_middleware(
        cache_class=cache_class,
        rpc_whitelist={'fake_endpoint'},
    ))

    assert w3.manager.request_blocking('fake_endpoint', [1]) == 'value-a'


def test_latest_block_based_cache_middleware_populates_cache(w3):
    result = w3.manager.request_blocking('fake_endpoint', [])

    assert w3.manager.request_blocking('fake_endpoint', []) == result
    assert w3.manager.request_blocking('fake_endpoint', [1]) != result


def test_latest_block_based_cache_middleware_busts_cache(w3, mocker):
    result = w3.manager.request_blocking('fake_endpoint', [])

    assert w3.manager.request_blocking('fake_endpoint', []) == result
    w3.testing.mine()

    # should still be cached for at least 1 second.  This also verifies that
    # the middleware caches the latest block based on the block time.
    assert w3.manager.request_blocking('fake_endpoint', []) == result

    mocker.patch('time.time', return_value=time.time() + 5)

    assert w3.manager.request_blocking('fake_endpoint', []) != result


def test_latest_block_cache_middleware_does_not_cache_bad_responses(
        w3_base,
        block_data_middleware,
        latest_block_based_cache_middleware):
    counter = itertools.count()
    w3 = w3_base

    def result_cb(method, params):
        next(counter)
        return None

    w3 = w3_base
    w3.middleware_stack.add(block_data_middleware)
    w3.middleware_stack.add(construct_result_generator_middleware({
        'fake_endpoint': result_cb,
    }))
    w3.middleware_stack.add(latest_block_based_cache_middleware)

    w3.manager.request_blocking('fake_endpoint', [])
    w3.manager.request_blocking('fake_endpoint', [])

    assert next(counter) == 2


def test_latest_block_cache_middleware_does_not_cache_error_response(
        w3_base,
        block_data_middleware,
        latest_block_based_cache_middleware):
    counter = itertools.count()
    w3 = w3_base

    def error_cb(method, params):
        next(counter)
        return "the error message"

    w3.middleware_stack.add(block_data_middleware)
    w3.middleware_stack.add(construct_error_generator_middleware({
        'fake_endpoint': error_cb,
    }))
    w3.middleware_stack.add(latest_block_based_cache_middleware)

    with pytest.raises(ValueError):
        w3.manager.request_blocking('fake_endpoint', [])
    with pytest.raises(ValueError):
        w3.manager.request_blocking('fake_endpoint', [])

    assert next(counter) == 2



from web3 import Web3
from web3.providers import (
    AutoProvider,
    BaseProvider,
)


class ConnectedProvider(BaseProvider):
    def isConnected(self):
        return True


class DisconnectedProvider(BaseProvider):
    def isConnected(self):
        return False


def test_isConnected_connected():
    """
    Web3.isConnected() returns True when connected to a node.
    """
    web3 = Web3(ConnectedProvider())
    assert web3.isConnected() is True


def test_isConnected_disconnected():
    """
    Web3.isConnected() returns False when configured with a provider
    that's not connected to a node.
    """
    web3 = Web3(DisconnectedProvider())
    assert web3.isConnected() is False


def test_autoprovider_detection():
    def no_provider():
        return None

    def must_not_call():
        assert False

    auto = AutoProvider([
        no_provider,
        DisconnectedProvider,
        ConnectedProvider,
        must_not_call,
    ])

    w3 = Web3(auto)

    assert w3.isConnected()

    assert isinstance(auto._active_provider, ConnectedProvider)



import pytest

from web3.utils.abi import (
    merge_args_and_kwargs,
)

FUNCTION_ABI = {
    "constant": False,
    "inputs": [
        {"name": "a", "type": "int256"},
        {"name": "b", "type": "int256"},
        {"name": "c", "type": "int256"},
        {"name": "d", "type": "int256"},
    ],
    "name": "testFn",
    "outputs": [],
    "type": "function",
}


def test_error_when_invalid_args_kwargs_combo_provided():
    with pytest.raises(TypeError):
        merge_args_and_kwargs(GENERATED_FUNCTION_ABI, (1, 2,), {'a': 1, 'b': 2})


@pytest.mark.parametrize(
    'args,kwargs,expected_args',
    (
        ((1, 4, 2, 3), {}, (1, 4, 2, 3)),
        ((1, 4, 2), {'d': 3}, (1, 4, 2, 3)),
        ((1, 4), {'d': 3, 'c': 2}, (1, 4, 2, 3)),
        ((1,), {'d': 3, 'b': 4, 'c': 2}, (1, 4, 2, 3)),
        (tuple(), {'d': 3, 'b': 4, 'a': 1, 'c': 2}, (1, 4, 2, 3)),
    ),
)
def test_merging_of_args_and_kwargs(args, kwargs, expected_args):
    actual_args = merge_args_and_kwargs(FUNCTION_ABI, args, kwargs)
    assert actual_args == expected_args


NO_INPUTS_FUNCTION_ABI = {
    "constant": False,
    "inputs": [],
    "name": "testFn",
    "outputs": [],
    "type": "function",
}


def test_merging_of_args_and_kwargs_with_no_inputs():
    actual = merge_args_and_kwargs(NO_INPUTS_FUNCTION_ABI, tuple(), {})
    assert actual == tuple()


GENERATED_FUNCTION_ABI = {
    "constant": False,
    "inputs": [
        {"name": "", "type": "int256"},
        {"name": "", "type": "int256"},
    ],
    "name": "testFn",
    "outputs": [],
    "type": "function",
}


def test_kwargs_is_disallowed_when_merging_with_unnamed_inputs():
    with pytest.raises(TypeError):
        merge_args_and_kwargs(GENERATED_FUNCTION_ABI, tuple(), {'x': 1, 'y': 2})


def test_args_works_when_merging_with_unnamed_inputs():
    actual = merge_args_and_kwargs(GENERATED_FUNCTION_ABI, (1, 2), {})
    assert actual == (1, 2)


DUPLICATE_NAMES_FUNCTION_ABI = {
    "constant": False,
    "inputs": [
        {"name": "b", "type": "int256"},
        {"name": "b", "type": "int256"},
        {"name": "a", "type": "int256"},
    ],
    "name": "testFn",
    "outputs": [],
    "type": "function",
}


def test_args_allowed_when_duplicate_named_inputs():
    actual = merge_args_and_kwargs(DUPLICATE_NAMES_FUNCTION_ABI, (1, 2, 3), {})
    assert actual == (1, 2, 3)


def test_kwargs_not_allowed_for_duplicate_input_names():
    with pytest.raises(TypeError):
        merge_args_and_kwargs(DUPLICATE_NAMES_FUNCTION_ABI, (1,), {'a': 2, 'b': 3})


def test_kwargs_allowed_if_no_intersections_with_duplicate_input_names():
    with pytest.raises(TypeError):
        merge_args_and_kwargs(DUPLICATE_NAMES_FUNCTION_ABI, (1,), {'a': 2, 'b': 3})



import itertools
import pytest


@pytest.fixture
def middleware_factory():
    unique_ids = itertools.count()

    def factory(key=None):
        if key is None:
            key = next(unique_ids)
        key = str(key)

        class Wrapper:
            def __repr__(self):
                return 'middleware-' + key

            def __call__(self, make_request, web3):
                def middleware_fn(method, params):
                    params.append(key)
                    method = "|".join((method, key))
                    response = make_request(method, params)
                    response['result']['middlewares'].append(key)
                    return response
                return middleware_fn
        return Wrapper()
    return factory



import contextlib
import json
import os
import pprint
import shutil
import sys
import time

from eth_utils import (
    to_text,
)

import common
import go_ethereum
from web3 import Web3
from web3.utils.toolz import (
    merge,
)

CHAIN_CONFIG = {
    "name": "CrossClient",
    "dataDir": "CrossClient",
    "engine": {
        "Ethash": {
            "params": {
                "minimumDifficulty": "0x020000",
                "difficultyBoundDivisor": "0x0800",
                "durationLimit": "0x0d",
                "blockReward": "0x4563918244F40000",
                "homesteadTransition": 0,
                "eip150Transition": 0,
                "eip160Transition": 10,
                "eip161abcTransition": 10,
                "eip161dTransition": 10
            }
        }
    },
    "params": {
        "gasLimitBoundDivisor": "0x0400",
        "registrar": "0x81a4b044831c4f12ba601adb9274516939e9b8a2",
        "eip155Transition": 10,
        "accountStartNonce": "0x0",
        "maximumExtraDataSize": "0x20",
        "minGasLimit": "0x1388",
        "networkID": "0x539",
        "eip98Transition": "0x7fffffffffffffff"
    },
    "genesis": {
        "seal": {
            "ethereum": {
                "nonce": "0x0000000000000042",
                "mixHash": "0x0000000000000000000000000000000000000000000000000000000000000000"
            }
        },
        "difficulty": "0x10000",
        "author": common.COINBASE,
        "timestamp": "0x00",
        "parentHash": "0x0000000000000000000000000000000000000000000000000000000000000000",
        "extraData": "0x3535353535353535353535353535353535353535353535353535353535353535",
        "gasLimit": "0x1000000"
    },
    "accounts": {
        common.COINBASE: {
            "balance": "1000000000000000000000000000",
            "nonce": "0",
            "builtin": {
                "name": "ecrecover", "pricing": {"linear": {"base": 3000, "word": 0}}
            }
        },
        common.UNLOCKABLE_ACCOUNT: {
            "balance": "1000000000000000000000000000",
            "nonce": "0",
            "builtin": {
                "name": "sha256", "pricing": {"linear": {"base": 60, "word": 12}}
            }
        },
        common.RAW_TXN_ACCOUNT: {
            "balance": "1000000000000000000000000000",
            "nonce": "0",
            "builtin": {
                "name": "ripemd160",
                "pricing": {"linear": {"base": 600, "word": 120}}
            }
        }
    }
}


def get_parity_binary():
    return 'parity'


@contextlib.contextmanager
def get_parity_process(
        parity_binary,
        datadir,
        ipc_path,
        keys_path,
        chain_config_file_path,
        parity_port):

    run_command = (
        parity_binary,
        '--base-path', datadir,
        '--ipc-path', ipc_path,
        '--no-ws',
        '--no-ui',
        '--no-warp',
        '--chain', chain_config_file_path,
        '--keys-path', keys_path,
        '--rpcapi', 'all',
        '--rpcport', parity_port,
        # '--author', common.COINBASE[2:],
    )
    print(' '.join(run_command))
    try:
        proc = common.get_process(run_command)
        yield proc
    finally:
        common.kill_proc_gracefully(proc)
        output, errors = proc.communicate()
        print(
            "Parity Process Exited:\n"
            "stdout:{0}\n\n"
            "stderr:{1}\n\n".format(
                to_text(output),
                to_text(errors),
            )
        )


@contextlib.contextmanager
def parity_export_blocks_process(
        parity_binary,
        datadir,
        chain_config_file_path,
        parity_port):

    run_command = (
        parity_binary,
        'export',
        'blocks', os.path.join(datadir, 'blocks_export.rlp'),
        '--base-path', datadir,
        '--no-ws',
        '--no-ui',
        '--no-warp',
        '--chain', chain_config_file_path,
        '--rpcapi', 'all',
        '--rpcport', parity_port,
        # '--author', common.COINBASE[2:],
    )
    print(' '.join(run_command))
    try:
        proc = common.get_process(run_command)
        yield proc
    finally:
        common.kill_proc_gracefully(proc)
        output, errors = proc.communicate()
        print(
            "Parity Process Exited:\n"
            "stdout:{0}\n\n"
            "stderr:{1}\n\n".format(
                to_text(output),
                to_text(errors),
            )
        )


def generate_parity_fixture(destination_dir):
    """
    The parity fixture generation strategy is to start a geth client with
    existing fixtures copied into a temp datadir.  Then a parity client
    is started is peered with the geth client.
    """
    with contextlib.ExitStack() as stack:

        geth_datadir = stack.enter_context(common.tempdir())

        geth_port = common.get_open_port()

        geth_ipc_path_dir = stack.enter_context(common.tempdir())
        geth_ipc_path = os.path.join(geth_ipc_path_dir, 'geth.ipc')

        geth_keystore_dir = os.path.join(geth_datadir, 'keystore')
        common.ensure_path_exists(geth_keystore_dir)
        geth_keyfile_path = os.path.join(geth_keystore_dir, common.KEYFILE_FILENAME)
        with open(geth_keyfile_path, 'w') as keyfile:
            keyfile.write(common.KEYFILE_DATA)

        genesis_file_path = os.path.join(geth_datadir, 'genesis.json')
        with open(genesis_file_path, 'w') as genesis_file:
            genesis_file.write(json.dumps(common.GENESIS_DATA))

        stack.enter_context(
            common.get_geth_process(
                common.get_geth_binary(),
                geth_datadir,
                genesis_file_path,
                geth_ipc_path,
                geth_port,
                str(CHAIN_CONFIG['params']['networkID']))
        )
        # set up fixtures
        common.wait_for_socket(geth_ipc_path)
        web3_geth = Web3(Web3.IPCProvider(geth_ipc_path))
        chain_data = go_ethereum.setup_chain_state(web3_geth)
        fixture_block_count = web3_geth.eth.blockNumber

        datadir = stack.enter_context(common.tempdir())

        keystore_dir = os.path.join(datadir, 'keys')
        os.makedirs(keystore_dir, exist_ok=True)
        parity_keyfile_path = os.path.join(keystore_dir, common.KEYFILE_FILENAME)
        with open(parity_keyfile_path, 'w') as keyfile:
            keyfile.write(common.KEYFILE_DATA)

        chain_config_file_path = os.path.join(datadir, 'chain_config.json')
        with open(chain_config_file_path, 'w') as chain_file:
            chain_file.write(json.dumps(CHAIN_CONFIG))

        parity_ipc_path_dir = stack.enter_context(common.tempdir())
        parity_ipc_path = os.path.join(parity_ipc_path_dir, 'jsonrpc.ipc')

        parity_port = common.get_open_port()
        parity_binary = get_parity_binary()

        parity_proc = stack.enter_context(get_parity_process(  # noqa: F841
            parity_binary=parity_binary,
            datadir=datadir,
            ipc_path=parity_ipc_path,
            keys_path=keystore_dir,
            chain_config_file_path=chain_config_file_path,
            parity_port=parity_port,
        ))

        common.wait_for_socket(parity_ipc_path)
        web3 = Web3(Web3.IPCProvider(parity_ipc_path))

        time.sleep(10)
        connect_nodes(web3, web3_geth)
        wait_for_chain_sync(web3, fixture_block_count)

        static_data = {
            'raw_txn_account': common.RAW_TXN_ACCOUNT,
            'keyfile_pw': common.KEYFILE_PW,
        }
        pprint.pprint(merge(chain_data, static_data))

        shutil.copytree(datadir, destination_dir)

        parity_proc = stack.enter_context(parity_export_blocks_process(  # noqa: F841
            parity_binary=parity_binary,
            datadir=destination_dir,
            chain_config_file_path=os.path.join(destination_dir, 'chain_config.json'),
            parity_port=parity_port,
        ))


def connect_nodes(w3_parity, w3_secondary):
    parity_peers = w3_parity.parity.netPeers()
    parity_enode = w3_parity.parity.enode()
    secondary_node_info = w3_secondary.admin.nodeInfo
    if secondary_node_info['id'] not in (node.get('id', tuple()) for node in parity_peers['peers']):
        w3_secondary.admin.addPeer(parity_enode)


def wait_for_chain_sync(web3, target):
    start_time = time.time()
    while time.time() < start_time + 120:
        current_block_number = web3.eth.blockNumber
        if current_block_number >= target:
            break
        else:
            time.sleep(0.1)
    else:
        raise ValueError("Not synced after wait period")


if __name__ == '__main__':
    fixture_dir = sys.argv[1]
    generate_parity_fixture(fixture_dir)



import os
import pytest

from tests.integration.common import (
    MiscWebsocketTest,
)
from tests.integration.utils import (
    wait_for_ws,
)
from web3 import Web3
from web3.utils.module_testing import (
    NetModuleTest,
    VersionModuleTest,
)

from .common import (
    ParityEthModuleTest,
    ParityPersonalModuleTest,
    ParityTraceModuleTest,
    ParityWeb3ModuleTest,
    get_open_port,
)


@pytest.fixture(scope="module")
def ws_port():
    return get_open_port()


@pytest.fixture(scope="module")
def endpoint_uri(ws_port):
    return 'ws://localhost:{0}'.format(ws_port)


@pytest.fixture(scope="module")
def parity_command_arguments(
    parity_import_blocks_process,
    parity_binary,
    datadir,
    passwordfile,
    author,
    ws_port
):
    return (
        parity_binary,
        '--chain', os.path.join(datadir, 'chain_config.json'),
        '--base-path', datadir,
        '--unlock', author,
        '--password', passwordfile,
        '--ws-port', ws_port,
        '--ws-origins', '*',
        '--no-ipc',
        '--no-jsonrpc',
    )


@pytest.fixture(scope="module")
def parity_import_blocks_command(parity_binary, ws_port, datadir, passwordfile):
    return (
        parity_binary,
        'import', os.path.join(datadir, 'blocks_export.rlp'),
        '--chain', os.path.join(datadir, 'chain_config.json'),
        '--base-path', datadir,
        '--password', passwordfile,
        '--ws-port', str(ws_port),
        '--ws-origins', '*',
        '--no-ipc',
        '--no-jsonrpc',
        '--tracing', 'on',
    )


@pytest.fixture(scope="module")  # noqa: F811
def web3(parity_process, endpoint_uri, event_loop):
    event_loop.run_until_complete(wait_for_ws(endpoint_uri, event_loop))
    _web3 = Web3(Web3.WebsocketProvider(endpoint_uri))
    return _web3


class TestParityWeb3ModuleTest(ParityWeb3ModuleTest):
    pass


class TestParityEthModuleTest(ParityEthModuleTest):
    pass


class TestParityVersionModule(VersionModuleTest):
    pass


class TestParityNetModule(NetModuleTest):
    pass


class TestParityPersonalModuleTest(ParityPersonalModuleTest):
    pass


class TestParityTraceModuleTest(ParityTraceModuleTest):
    pass


class TestMiscWebsocketTest(MiscWebsocketTest):
    pass



# coding=utf-8

import pytest

from eth_account.messages import (
    defunct_hash_message,
)
from eth_utils import (
    is_checksum_address,
)
from hexbytes import (
    HexBytes,
)

from web3 import (
    Account,
    Web3,
)
from web3.providers.eth_tester import (
    EthereumTesterProvider,
)
from web3.utils.encoding import (
    to_bytes,
    to_hex,
)
from web3.utils.toolz import (
    dissoc,
)

# from https://github.com/ethereum/tests/blob/3930ca3a9a377107d5792b3e7202f79c688f1a67/BasicTests/txtest.json # noqa: 501
ETH_TEST_TRANSACTIONS = [
    {
        "chainId": None,
        "key": "c85ef7d79691fe79573b1a7064c19c1a9819ebdbd1faaab1a8ec92344438aaf4",
        "nonce": 0,
        "gasPrice": 1000000000000,
        "gas": 10000,
        "to": "0x13978aee95f38490e9769C39B2773Ed763d9cd5F",
        "value": 10000000000000000,
        "data": "",
        "unsigned": "eb8085e8d4a510008227109413978aee95f38490e9769c39b2773ed763d9cd5f872386f26fc1000080808080",  # noqa: 501
        "signed": "f86b8085e8d4a510008227109413978aee95f38490e9769c39b2773ed763d9cd5f872386f26fc10000801ba0eab47c1a49bf2fe5d40e01d313900e19ca485867d462fe06e139e3a536c6d4f4a014a569d327dcda4b29f74f93c0e9729d2f49ad726e703f9cd90dbb0fbf6649f1"  # noqa: 501
    },
    {
        "chainId": None,
        "key": "c87f65ff3f271bf5dc8643484f66b200109caffe4bf98c4cb393dc35740b28c0",
        "nonce": 0,
        "gasPrice": 1000000000000,
        "gas": 10000,
        "to": "",
        "value": 0,
        "data": "6025515b525b600a37f260003556601b596020356000355760015b525b54602052f260255860005b525b54602052f2",  # noqa: 501
        "unsigned": "f83f8085e8d4a510008227108080af6025515b525b600a37f260003556601b596020356000355760015b525b54602052f260255860005b525b54602052f2808080",  # noqa: 501
        "signed": "f87f8085e8d4a510008227108080af6025515b525b600a37f260003556601b596020356000355760015b525b54602052f260255860005b525b54602052f21ba05afed0244d0da90b67cf8979b0f246432a5112c0d31e8d5eedd2bc17b171c694a0bb1035c834677c2e1185b8dc90ca6d1fa585ab3d7ef23707e1a497a98e752d1b"  # noqa: 501
    }
]


@pytest.fixture
def PRIVATE_BYTES():
    return b'unicorns' * 4


@pytest.fixture
def PRIVATE_BYTES_ALT(PRIVATE_BYTES):
    return b'rainbows' * 4


@pytest.fixture
def web3js_key():
    return '0x4c0883a69102937d6231471b5dbb6204fe5129617082792ae468d01a3f362318'


@pytest.fixture
def web3js_password():
    return 'test!'


@pytest.fixture(params=['instance', 'class'])
def acct(request, web3):
    if request.param == 'instance':
        return web3.eth.account
    elif request.param == 'class':
        return Account
    raise Exception('Unreachable!')


@pytest.fixture()
def w3():
    return Web3(EthereumTesterProvider())


def test_eth_account_create_variation(acct):
    account1 = acct.create()
    account2 = acct.create()
    assert account1 != account2


def test_eth_account_privateKeyToAccount_reproducible(acct, PRIVATE_BYTES):
    account1 = acct.privateKeyToAccount(PRIVATE_BYTES)
    account2 = acct.privateKeyToAccount(PRIVATE_BYTES)
    assert bytes(account1) == PRIVATE_BYTES
    assert bytes(account1) == bytes(account2)
    assert isinstance(str(account1), str)


def test_eth_account_privateKeyToAccount_diverge(acct, PRIVATE_BYTES, PRIVATE_BYTES_ALT):
    account1 = acct.privateKeyToAccount(PRIVATE_BYTES)
    account2 = acct.privateKeyToAccount(PRIVATE_BYTES_ALT)
    assert bytes(account2) == PRIVATE_BYTES_ALT
    assert bytes(account1) != bytes(account2)


def test_eth_account_privateKeyToAccount_seed_restrictions(acct):
    with pytest.raises(ValueError):
        acct.privateKeyToAccount(b'')
    with pytest.raises(ValueError):
        acct.privateKeyToAccount(b'\xff' * 31)
    with pytest.raises(ValueError):
        acct.privateKeyToAccount(b'\xff' * 33)


def test_eth_account_privateKeyToAccount_properties(acct, PRIVATE_BYTES):
    account = acct.privateKeyToAccount(PRIVATE_BYTES)
    assert callable(account.signHash)
    assert callable(account.signTransaction)
    assert is_checksum_address(account.address)
    assert account.address == '0xa79F6f349C853F9Ea0B29636779ae3Cb4E3BA729'
    assert account.privateKey == PRIVATE_BYTES


def test_eth_account_create_properties(acct):
    account = acct.create()
    assert callable(account.signHash)
    assert callable(account.signTransaction)
    assert is_checksum_address(account.address)
    assert isinstance(account.privateKey, bytes) and len(account.privateKey) == 32


def test_eth_account_recover_transaction_example(acct):
    raw_tx_hex = '0xf8640d843b9aca00830e57e0945b2063246f2191f18f2675cedb8b28102e957458018025a00c753084e5a8290219324c1a3a86d4064ded2d15979b1ea790734aaa2ceaafc1a0229ca4538106819fd3a5509dd383e8fe4b731c6870339556a5c06feb9cf330bb'  # noqa: E501
    from_account = acct.recoverTransaction(raw_tx_hex)
    assert from_account == '0xFeC2079e80465cc8C687fFF9EE6386ca447aFec4'


def test_eth_account_recover_transaction_with_literal(acct):
    raw_tx = 0xf8640d843b9aca00830e57e0945b2063246f2191f18f2675cedb8b28102e957458018025a00c753084e5a8290219324c1a3a86d4064ded2d15979b1ea790734aaa2ceaafc1a0229ca4538106819fd3a5509dd383e8fe4b731c6870339556a5c06feb9cf330bb  # noqa: E501
    from_account = acct.recoverTransaction(raw_tx)
    assert from_account == '0xFeC2079e80465cc8C687fFF9EE6386ca447aFec4'


def test_eth_account_recover_message(acct):
    v, r, s = (
        28,
        '0xe6ca9bba58c88611fad66a6ce8f996908195593807c4b38bd528d2cff09d4eb3',
        '0x3e5bfbbf4d3e39b1a2fd816a7680c19ebebaf3a141b239934ad43cb33fcec8ce',
    )
    message = "I♥SF"
    message_hash = defunct_hash_message(text=message)
    from_account = acct.recoverHash(message_hash, vrs=(v, r, s))
    assert from_account == '0x5ce9454909639D2D17A3F753ce7d93fa0b9aB12E'


@pytest.mark.parametrize(
    'signature_bytes',
    [
        # test signature bytes with standard v (0 in this case)
        b'\x0cu0\x84\xe5\xa8)\x02\x192L\x1a:\x86\xd4\x06M\xed-\x15\x97\x9b\x1e\xa7\x90sJ\xaa,\xea\xaf\xc1"\x9c\xa4S\x81\x06\x81\x9f\xd3\xa5P\x9d\xd3\x83\xe8\xfeKs\x1chp3\x95V\xa5\xc0o\xeb\x9c\xf30\xbb\x00',  # noqa: E501
        # test signature bytes with chain-naive v (27 in this case)
        b'\x0cu0\x84\xe5\xa8)\x02\x192L\x1a:\x86\xd4\x06M\xed-\x15\x97\x9b\x1e\xa7\x90sJ\xaa,\xea\xaf\xc1"\x9c\xa4S\x81\x06\x81\x9f\xd3\xa5P\x9d\xd3\x83\xe8\xfeKs\x1chp3\x95V\xa5\xc0o\xeb\x9c\xf30\xbb\x1b',  # noqa: E501
    ],
    ids=['test_sig_bytes_standard_v', 'test_sig_bytes_chain_naive_v']
)
def test_eth_account_recover_signature_bytes(acct, signature_bytes):
    msg_hash = b'\xbb\r\x8a\xba\x9f\xf7\xa1<N,s{i\x81\x86r\x83{\xba\x9f\xe2\x1d\xaa\xdd\xb3\xd6\x01\xda\x00\xb7)\xa1'  # noqa: E501
    from_account = acct.recoverHash(msg_hash, signature=signature_bytes)
    assert from_account == '0xFeC2079e80465cc8C687fFF9EE6386ca447aFec4'


def test_eth_account_recover_vrs(acct):
    v, r, s = (
        27,
        5634810156301565519126305729385531885322755941350706789683031279718535704513,
        15655399131600894366408541311673616702363115109327707006109616887384920764603,
    )
    msg_hash = b'\xbb\r\x8a\xba\x9f\xf7\xa1<N,s{i\x81\x86r\x83{\xba\x9f\xe2\x1d\xaa\xdd\xb3\xd6\x01\xda\x00\xb7)\xa1'  # noqa: E501
    from_account = acct.recoverHash(msg_hash, vrs=(v, r, s))
    assert from_account == '0xFeC2079e80465cc8C687fFF9EE6386ca447aFec4'

    from_account = acct.recoverHash(msg_hash, vrs=map(to_hex, (v, r, s)))
    assert from_account == '0xFeC2079e80465cc8C687fFF9EE6386ca447aFec4'


def test_eth_account_recover_vrs_standard_v(acct):
    v, r, s = (
        0,
        5634810156301565519126305729385531885322755941350706789683031279718535704513,
        15655399131600894366408541311673616702363115109327707006109616887384920764603,
    )
    msg_hash = b'\xbb\r\x8a\xba\x9f\xf7\xa1<N,s{i\x81\x86r\x83{\xba\x9f\xe2\x1d\xaa\xdd\xb3\xd6\x01\xda\x00\xb7)\xa1'  # noqa: E501
    from_account = acct.recoverHash(msg_hash, vrs=(v, r, s))
    assert from_account == '0xFeC2079e80465cc8C687fFF9EE6386ca447aFec4'


@pytest.mark.parametrize(
    'message, key, expected_bytes, expected_hash, v, r, s, signature',
    (
        (
            'Some data',
            '0x4c0883a69102937d6231471b5dbb6204fe5129617082792ae468d01a3f362318',
            b'Some data',
            HexBytes('0x1da44b586eb0729ff70a73c326926f6ed5a25f5b056e7f47fbc6e58d86871655'),
            28,
            83713930994764734002432606962255364472443135907807238282514898577139886061053,
            43435997768575461196683613590576722655951133545204789519877940758262837256233,
            HexBytes('0xb91467e570a6466aa9e9876cbcd013baba02900b8979d43fe208a4a4f339f5fd6007e74cd82e037b800186422fc2da167c747ef045e5d18a5f5d4300f8e1a0291c'),  # noqa: E501
        ),
        (
            '10284',
            '0x4c0883a69102937d6231471b5dbb6204fe5129617082792ae468d01a3f362318',
            b'10284',
            HexBytes('0x0a162a5efbba02f38db3114531c8acba39fe676f09f7e471d93e8a06c471821c'),
            27,
            143748089818580655331728101695676826715814583506606354117109114714663470502,
            227853308212209543997879651656855994238138056366857653269155208245074180053,
            HexBytes('0x00515bc8fd32264e21ec0820e8c5123ed58c1195c9ea17cb018b1ad4073cc5a60080f5dcec397a5a8c523082bfa41771568903aa554ec06ba8475ca9050fb7d51b'),  # noqa: E501
        ),

    ),
    ids=['web3js_example', '31byte_r_and_s'],
)
def test_eth_account_sign(acct, message, key, expected_bytes, expected_hash, v, r, s, signature):
    message_hash = defunct_hash_message(text=message)
    assert message_hash == expected_hash

    signed = acct.signHash(message_hash, private_key=key)
    assert signed.messageHash == expected_hash
    assert signed.v == v
    assert signed.r == r
    assert signed.s == s
    assert signed.signature == signature

    account = acct.privateKeyToAccount(key)
    assert account.signHash(message_hash) == signed


@pytest.mark.parametrize(
    'txn, private_key, expected_raw_tx, tx_hash, r, s, v',
    (
        (
            {
                'to': '0xF0109fC8DF283027b6285cc889F5aA624EaC1F55',
                'value': 1000000000,
                'gas': 2000000,
                'gasPrice': 234567897654321,
                'nonce': 0,
                'chainId': 1
            },
            '0x4c0883a69102937d6231471b5dbb6204fe5129617082792ae468d01a3f362318',
            HexBytes('0xf86a8086d55698372431831e848094f0109fc8df283027b6285cc889f5aa624eac1f55843b9aca008025a009ebb6ca057a0535d6186462bc0b465b561c94a295bdb0621fc19208ab149a9ca0440ffd775ce91a833ab410777204d5341a6f9fa91216a6f3ee2c051fea6a0428'),  # noqa: E501
            HexBytes('0xd8f64a42b57be0d565f385378db2f6bf324ce14a594afc05de90436e9ce01f60'),
            4487286261793418179817841024889747115779324305375823110249149479905075174044,
            30785525769477805655994251009256770582792548537338581640010273753578382951464,
            37,
        ),
        (
            {
                'to': '0xF0109fC8DF283027b6285cc889F5aA624EaC1F55',
                'value': 0,
                'gas': 31853,
                'gasPrice': 0,
                'nonce': 0,
                'chainId': 1
            },
            '0x4c0883a69102937d6231471b5dbb6204fe5129617082792ae468d01a3f362318',
            HexBytes('0xf85d8080827c6d94f0109fc8df283027b6285cc889f5aa624eac1f558080269f22f17b38af35286ffbb0c6376c86ec91c20ecbad93f84913a0cc15e7580cd99f83d6e12e82e3544cb4439964d5087da78f74cefeec9a450b16ae179fd8fe20'),  # noqa: E501
            HexBytes('0xb0c5e2c6b29eeb0b9c1d63eaa8b0f93c02ead18ae01cb7fc795b0612d3e9d55a'),
            61739443115046231975538240097110168545680205678104352478922255527799426265,
            232940010090391255679819602567388136081614408698362277324138554019997613600,
            38,
        ),
    ),
    ids=['web3js_example', '31byte_r_and_s'],
)
def test_eth_account_sign_transaction(acct, txn, private_key, expected_raw_tx, tx_hash, r, s, v):
    signed = acct.signTransaction(txn, private_key)
    assert signed.r == r
    assert signed.s == s
    assert signed.v == v
    assert signed.rawTransaction == expected_raw_tx
    assert signed.hash == tx_hash

    account = acct.privateKeyToAccount(private_key)
    assert account.signTransaction(txn) == signed


@pytest.mark.parametrize(
    'transaction_info',
    ETH_TEST_TRANSACTIONS,
)
def test_eth_account_sign_transaction_from_eth_test(acct, transaction_info):
    expected_raw_txn = transaction_info['signed']
    key = transaction_info['key']

    transaction = dissoc(transaction_info, 'signed', 'key', 'unsigned')

    # validate r, in order to validate the transaction hash
    # There is some ambiguity about whether `r` will always be deterministically
    # generated from the transaction hash and private key, mostly due to code
    # author's ignorance. The example test fixtures and implementations seem to agree, so far.
    # See ecdsa_raw_sign() in /eth_keys/backends/native/ecdsa.py
    signed = acct.signTransaction(transaction, key)
    assert signed.r == Web3.toInt(hexstr=expected_raw_txn[-130:-66])

    # confirm that signed transaction can be recovered to the sender
    expected_sender = acct.privateKeyToAccount(key).address
    assert acct.recoverTransaction(signed.rawTransaction) == expected_sender


@pytest.mark.parametrize(
    'transaction',
    ETH_TEST_TRANSACTIONS,
)
def test_eth_account_recover_transaction_from_eth_test(acct, transaction):
    raw_txn = transaction['signed']
    key = transaction['key']
    expected_sender = acct.privateKeyToAccount(key).address
    assert acct.recoverTransaction(raw_txn) == expected_sender


def test_eth_account_encrypt(acct, web3js_key, web3js_password):
    encrypted = acct.encrypt(web3js_key, web3js_password)

    assert encrypted['address'] == '2c7536e3605d9c16a7a3d7b1898e529396a65c23'
    assert encrypted['version'] == 3

    decrypted_key = acct.decrypt(encrypted, web3js_password)

    assert decrypted_key == to_bytes(hexstr=web3js_key)


def test_eth_account_prepared_encrypt(acct, web3js_key, web3js_password):
    account = acct.privateKeyToAccount(web3js_key)
    encrypted = account.encrypt(web3js_password)

    assert encrypted['address'] == '2c7536e3605d9c16a7a3d7b1898e529396a65c23'
    assert encrypted['version'] == 3

    decrypted_key = acct.decrypt(encrypted, web3js_password)

    assert decrypted_key == to_bytes(hexstr=web3js_key)


@pytest.mark.xfail
@pytest.mark.parametrize(
    'expected_txn, raw_tx, expected_tx_hash, r, s, v',
    (
        (
            {
                'to': '0xF0109fC8DF283027b6285cc889F5aA624EaC1F55',
                'value': 1000000000,
                'gas': 2000000,
                'gasPrice': 234567897654321,
                'nonce': 0,
                'chainId': 1
            },
            HexBytes('0xf86a8086d55698372431831e848094f0109fc8df283027b6285cc889f5aa624eac1f55843b9aca008025a009ebb6ca057a0535d6186462bc0b465b561c94a295bdb0621fc19208ab149a9ca0440ffd775ce91a833ab410777204d5341a6f9fa91216a6f3ee2c051fea6a0428'),  # noqa: E501
            HexBytes('0xd8f64a42b57be0d565f385378db2f6bf324ce14a594afc05de90436e9ce01f60'),
            4487286261793418179817841024889747115779324305375823110249149479905075174044,
            30785525769477805655994251009256770582792548537338581640010273753578382951464,
            37,
        ),
        (
            {
                'to': '0xF0109fC8DF283027b6285cc889F5aA624EaC1F55',
                'value': 0,
                'gas': 31853,
                'gasPrice': 0,
                'nonce': 0,
                'chainId': 1
            },
            HexBytes('0xf85d8080827c6d94f0109fc8df283027b6285cc889f5aa624eac1f558080269f22f17b38af35286ffbb0c6376c86ec91c20ecbad93f84913a0cc15e7580cd99f83d6e12e82e3544cb4439964d5087da78f74cefeec9a450b16ae179fd8fe20'),  # noqa: E501
            HexBytes('0xb0c5e2c6b29eeb0b9c1d63eaa8b0f93c02ead18ae01cb7fc795b0612d3e9d55a'),
            61739443115046231975538240097110168545680205678104352478922255527799426265,
            232940010090391255679819602567388136081614408698362277324138554019997613600,
            38,
        ),
    ),
    ids=['web3js_example', '31byte_r_and_s'],
)
def test_eth_account_sign_and_send_EIP155_transaction_to_eth_tester(
        w3,
        expected_txn,
        raw_tx,
        expected_tx_hash,
        r, s, v):
    actual_tx_hash = w3.eth.sendRawTransaction(raw_tx)
    assert actual_tx_hash == expected_tx_hash
    actual_txn = w3.eth.getTransaction(actual_tx_hash)
    for key in ('to', 'nonce', 'gas', 'gasPrice', 'value', ):
        assert actual_txn[key] == expected_txn[key]
    assert actual_txn.r == r
    assert actual_txn.s == s
    assert actual_txn.v == v



import pytest

from eth_utils import (
    to_bytes,
)

from web3.exceptions import (
    InvalidAddress,
)
from web3.utils.validation import (
    validate_abi,
    validate_abi_type,
    validate_abi_value,
    validate_address,
)

ABI = [
    {
        "constant": False,
        "inputs": [],
        "name": "func_1",
        "outputs": [],
        "type": "function",
    },
]

MALFORMED_ABI_1 = "NON-LIST ABI"
MALFORMED_ABI_2 = [5, {"test": "value"}, True]
MALFORMED_SELECTOR_COLLISION_ABI = [
    {
        'constant': False,
        'inputs': [{'name': 'input', 'type': 'uint256'}],
        'name': 'blockHashAmphithyronVersify',
        'outputs': [{'name': '', 'type': 'uint256'}],
        'payable': False,
        'stateMutability': 'nonpayable',
        'type': 'function'
    },
    {
        'constant': False,
        'inputs': [{'name': 'input', 'type': 'uint256'}],
        'name': 'blockHashAskewLimitary',
        'outputs': [{'name': '', 'type': 'uint256'}],
        'payable': False,
        'stateMutability': 'nonpayable',
        'type': 'function'
    }
]
MALFORMED_SIGNATURE_COLLISION_ABI = [
    {
        'constant': False,
        'inputs': [{'name': 'input', 'type': 'uint256'}],
        'name': 'blockHashAmphithyronVersify',
        'outputs': [{'name': '', 'type': 'uint256'}],
        'payable': False,
        'stateMutability': 'nonpayable',
        'type': 'function'
    },
    {
        'constant': False,
        'inputs': [{'name': 'input', 'type': 'uint256'}],
        'name': 'blockHashAmphithyronVersify',
        'outputs': [{'name': '', 'type': 'uint256'}],
        'payable': False,
        'stateMutability': 'nonpayable',
        'type': 'function'
    }
]

ADDRESS = '0xd3CdA913deB6f67967B99D67aCDFa1712C293601'
BYTES_ADDRESS = to_bytes(hexstr=ADDRESS)
PADDED_ADDRESS = '0x000000000000000000000000d3cda913deb6f67967b99d67acdfa1712c293601'
INVALID_CHECKSUM_ADDRESS = '0xd3CDA913deB6f67967B99D67aCDFa1712C293601'
NON_CHECKSUM_ADDRESS = '0xd3cda913deb6f67967b99d67acdfa1712c293601'
BYTES_ADDRESS_LEN_LT_20 = bytes(1) * 19
BYTES_ADDRESS_LEN_GT_20 = bytes(1) * 21


@pytest.mark.parametrize(
    'param,validation,expected',
    (
        (ABI, validate_abi, None),
        (MALFORMED_ABI_1, validate_abi, ValueError),
        (MALFORMED_ABI_2, validate_abi, ValueError),
        (MALFORMED_SELECTOR_COLLISION_ABI, validate_abi, ValueError),
        (MALFORMED_SIGNATURE_COLLISION_ABI, validate_abi, ValueError),
        (ADDRESS, validate_address, None),
        (BYTES_ADDRESS, validate_address, None),
        (PADDED_ADDRESS, validate_address, InvalidAddress),
        (INVALID_CHECKSUM_ADDRESS, validate_address, InvalidAddress),
        (NON_CHECKSUM_ADDRESS, validate_address, InvalidAddress),
        (BYTES_ADDRESS_LEN_LT_20, validate_address, InvalidAddress),
        (BYTES_ADDRESS_LEN_GT_20, validate_address, InvalidAddress),
        ("NotAddress", validate_address, InvalidAddress),
        (b'not string', validate_address, InvalidAddress),
        ('bool', validate_abi_type, None),
        ('bool[', validate_abi_type, ValueError),
        ('sbool', validate_abi_type, ValueError),
        ('stringx', validate_abi_type, ValueError),
        ('address', validate_abi_type, None),
    )
)
def test_validation(param, validation, expected):

    if isinstance(expected, type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            validation(param)
        return

    validation(param)


@pytest.mark.parametrize(
    'abi_type,value,expected',
    (
        ('string', True, TypeError),
        ('bool[]', [1, 3], TypeError),
        ('bool[]', [True, False], None),
        ('bool[][2]', [[True, False], [True, False, True]], None),
        ('bool[][2]', [[True, False], [True], [False]], TypeError),
        ('bool[3][]', [[True, False, False]], None),
        ('bool[3][]', [[True, False]], TypeError),
        ('bool[0]', [], TypeError),
        ('bool[0][1]', [[]], TypeError),
        ('uint8', -5, TypeError),
        ('int8', -5, None),
        ('address', "just a string", InvalidAddress),
        ('address', b'not even a string', InvalidAddress),
        ('address[][]', [[4, 5], [True]], TypeError),
        ('address[][]', [[ADDRESS]], None),
        ('address[2][]', [[ADDRESS], [ADDRESS, ADDRESS]], TypeError),
        ('address[2][1]', [[ADDRESS, ADDRESS]], None),
        ('address[2][1]', [[ADDRESS], [ADDRESS, ADDRESS]], TypeError),
        ('bytes', True, TypeError),
        ('bytes', "0x5402", None),
        ('bytes', "5402", TypeError),
        ('bytes', b'T\x02', None),
    )
)
def test_validate_abi_value(abi_type, value, expected):

    if isinstance(expected, type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            validate_abi_value(abi_type, value)
        return

    validate_abi_value(abi_type, value)



from web3.gas_strategies.rpc import (
    rpc_gas_price_strategy,
)


def test_default_rpc_gas_price_strategy(web3):
    assert rpc_gas_price_strategy(web3, {
        'to': '0x0',
        'value': 1
    }) == 1


def test_default_rpc_gas_price_strategy_callable_without_transaction(web3):
    assert rpc_gas_price_strategy(web3) == 1



import pytest

from hexbytes import (
    HexBytes,
)

from web3.utils.toolz import (
    compose,
    curry,
)

AMBIGUOUS_CONTRACT_ABI = [
    {
        'constant': False,
        'inputs': [{'name': 'input', 'type': 'uint256'}],
        'name': 'blockHashAmphithyronVersify',
        'outputs': [{'name': '', 'type': 'uint256'}],
        'payable': False,
        'stateMutability': 'nonpayable',
        'type': 'function'
    },
    {
        'constant': False,
        'inputs': [{'name': 'input', 'type': 'uint256'}, {'name': 'uselessFlag', 'type': 'bool'}],
        'name': 'identity',
        'outputs': [{'name': '', 'type': 'uint256'}],
        'payable': False,
        'stateMutability': 'nonpayable',
        'type': 'function'
    },
    {
        'constant': False,
        'inputs': [{'name': 'input', 'type': 'int256'}, {'name': 'uselessFlag', 'type': 'bool'}],
        'name': 'identity',
        'outputs': [{'name': '', 'type': 'int256'}],
        'payable': False,
        'stateMutability': 'nonpayable',
        'type': 'function'
    }
]


@pytest.fixture()
def string_contract(web3, StringContract, address_conversion_func):
    deploy_txn = StringContract.constructor("Caqalai").transact()
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert deploy_receipt is not None
    contract_address = address_conversion_func(deploy_receipt['contractAddress'])
    contract = StringContract(address=contract_address)
    assert contract.address == contract_address
    assert len(web3.eth.getCode(contract.address)) > 0
    return contract


map_repr = compose(list, curry(map, repr))


@pytest.mark.parametrize(
    'method,args,repr_func,expected',
    (
        (
            'all_functions',
            (),
            map_repr,
            [
                '<Function blockHashAmphithyronVersify(uint256)>',
                '<Function identity(uint256,bool)>',
                '<Function identity(int256,bool)>'
            ]
        ),
        (
            'get_function_by_signature',
            ('identity(uint256,bool)',),
            repr,
            '<Function identity(uint256,bool)>'
        ),
        (
            'find_functions_by_name',
            ('identity',),
            map_repr,
            [
                '<Function identity(uint256,bool)>',
                '<Function identity(int256,bool)>'
            ]
        ),
        (
            'get_function_by_name',
            ('blockHashAmphithyronVersify',),
            repr,
            '<Function blockHashAmphithyronVersify(uint256)>',
        ),
        (
            'get_function_by_selector',
            (b'\x00\x00\x00\x00',),
            repr,
            '<Function blockHashAmphithyronVersify(uint256)>',
        ),
        (
            'get_function_by_selector',
            (0x00000000,),
            repr,
            '<Function blockHashAmphithyronVersify(uint256)>',
        ),
        (
            'get_function_by_selector',
            ('0x00000000',),
            repr,
            '<Function blockHashAmphithyronVersify(uint256)>',
        ),
        (
            'find_functions_by_args',
            (1, True),
            map_repr,
            [
                '<Function identity(uint256,bool)>',
                '<Function identity(int256,bool)>'
            ]
        ),
        (
            'get_function_by_args',
            (1,),
            repr,
            '<Function blockHashAmphithyronVersify(uint256)>',
        ),
    ),
)
def test_find_or_get_functions_by_type(web3, method, args, repr_func, expected):
    contract = web3.eth.contract(abi=AMBIGUOUS_CONTRACT_ABI)
    function = getattr(contract, method)(*args)
    assert repr_func(function) == expected


@pytest.mark.parametrize(
    'method,args,expected_message,expected_error',
    (
        (
            'get_function_by_signature',
            ('identity(uint256, bool)', ),
            r'Function signature should not contain any spaces.*',
            ValueError
        ),
        (
            'get_function_by_name',
            ('identity', ),
            r'Found multiple functions with matching name*',
            ValueError
        ),
        (
            'get_function_by_name',
            ('undefined_function', ),
            r'Could not find any function with matching name',
            ValueError
        ),
        (
            'get_function_by_selector',
            (b'\x00' * (4 + 1), ),
            r'expected value of size 4 bytes. Got: %s bytes' % (4 + 1),
            ValueError
        ),
        (
            'get_function_by_args',
            (1, True),
            r'Found multiple functions with matching args*',
            ValueError
        ),
        (
            'get_function_by_args',
            (1,) * 50,
            r'Could not find any function with matching args',
            ValueError
        ),
    )
)
def test_functions_error_messages(web3, method, args, expected_message, expected_error):
    contract = web3.eth.contract(abi=AMBIGUOUS_CONTRACT_ABI)
    with pytest.raises(expected_error, match=expected_message):
        getattr(contract, method)(*args)


def test_contract_function_methods(string_contract):
    set_value_func = string_contract.get_function_by_signature('setValue(string)')
    get_value_func = string_contract.get_function_by_signature('getValue()')
    assert isinstance(set_value_func('Hello').transact(), HexBytes)
    assert get_value_func().call() == 'Hello'
    assert isinstance(set_value_func('Hello World').estimateGas(), int)
    assert isinstance(set_value_func('Hello World').buildTransaction(), dict)


def test_diff_between_fn_and_fn_called(string_contract):
    get_value_func = string_contract.get_function_by_signature('getValue()')
    get_value_func_called = get_value_func()
    assert get_value_func is not get_value_func_called
    assert repr(get_value_func) == '<Function getValue()>'
    assert repr(get_value_func_called) == '<Function getValue() bound to ()>'



from web3.manager import (
    RequestManager,
)
from web3.providers import (
    BaseProvider,
)


class DummyProvider(BaseProvider):
    def make_request(self, method, params):
        return {
            'result': {
                'method': method,
                'params': params,
                'middlewares': [],
            },
        }


def test_provider_property_setter_and_getter(middleware_factory):
    middleware_a = middleware_factory('middleware-A')
    middleware_b = middleware_factory('middleware-B')

    provider = DummyProvider()

    manager = RequestManager(None, provider, middlewares=[middleware_a, middleware_b])
    response = manager.request_blocking('init', ['init'])

    assert response['method'] == 'init|middleware-A|middleware-B'
    assert response['params'] == ['init', 'middleware-A', 'middleware-B']
    assert response['middlewares'] == ['middleware-B', 'middleware-A']




import pytest
from unittest.mock import (
    patch,
)

from ens import ENS
from web3 import Web3


def test_resolver_empty(ens):
    with patch.object(ens.ens, 'resolver', return_value=None):
        assert ens.resolver('') is None


@pytest.mark.parametrize(
    'address, expected_reverse',
    [
        (
            '0x1111111111111111111111111111111111111111',
            '1111111111111111111111111111111111111111.addr.reverse',
        ),
        (
            '0xBB9bc244D798123fDe783fCc1C72d3Bb8C189413',
            'bb9bc244d798123fde783fcc1c72d3bb8c189413.addr.reverse',
        ),
    ],
)
def test_reverse_domain(address, expected_reverse, address_conversion_func):
    address = address_conversion_func(address)
    assert ENS.reverse_domain(address) == expected_reverse


@pytest.mark.parametrize(
    'label, expected_hash',
    [
        ('eth', '0x4f5b812789fc606be1b3b16908db13fc7a9adf7ca72641f84d75b47069d3d7f0'),
        ('ETH', '0x4f5b812789fc606be1b3b16908db13fc7a9adf7ca72641f84d75b47069d3d7f0'),
        ('a.b', ValueError),
    ],
)
def test_labelhash(ens, label, expected_hash):
    if isinstance(expected_hash, type):
        with pytest.raises(expected_hash):
            ens.labelhash(label)
    else:
        labelhash = ens.labelhash(label)
        assert isinstance(labelhash, bytes)
        hash_hex = Web3.toHex(labelhash)
        assert hash_hex == expected_hash



import pytest

from web3 import Web3
from web3.exceptions import (
    InvalidAddress,
)
from web3.middleware import (  # noqa: F401
    construct_fixture_middleware,
    name_to_address_middleware,
)
from web3.providers.base import (
    BaseProvider,
)

NAME = "dump.eth"
ADDRESS = "0x0000000000000000000000000000000000000000"
BALANCE = 0


class TempENS():
    def __init__(self, name_addr_pairs):
        self.registry = dict(name_addr_pairs)

    def address(self, name, guess_tld=True):
        # no automated web3 usages should be guessing the TLD
        assert not guess_tld
        return self.registry.get(name, None)


@pytest.fixture
def w3():
    w3 = Web3(providers=[BaseProvider()], middlewares=[])
    w3.ens = TempENS({NAME: ADDRESS})
    w3.middleware_stack.add(name_to_address_middleware(w3))
    return w3


def test_pass_name_resolver(w3):
    return_chain_on_mainnet = construct_fixture_middleware({
        'net_version': '1',
    })
    return_balance = construct_fixture_middleware({
        'eth_getBalance': BALANCE
    })
    w3.middleware_stack.inject(return_chain_on_mainnet, layer=0)
    w3.middleware_stack.inject(return_balance, layer=0)
    assert w3.eth.getBalance(NAME) == BALANCE


def test_fail_name_resolver(w3):
    return_chain_on_mainnet = construct_fixture_middleware({
        'net_version': '2',
    })
    w3.middleware_stack.inject(return_chain_on_mainnet, layer=0)
    with pytest.raises(InvalidAddress, match='.*ethereum\.eth.*'):
        w3.eth.getBalance("ethereum.eth")
